//*****************************************************************************
//
//  Title:    WRTRSAMP.C
//
//  Purpose:  This is a sample program to show how to use the PC Support/400
//            Router API for Windows.  It requires a transaction program on
//            the host, which is included in this package.  Please read the
//            QIWSTOOL abstract for this package for a complete description
//            on how to create the host program.
//
//            This test driver exercises all of the router APIs.  You should
//            be aware that it almost always leaves a conversation in SEND
//            state, because the first thing it does for most of the menu items
//            is tell the host what is coming.  For several of the menu items,
//            many APPC operations are performed.
//
//            We didn't do much parameter checking for the dialog boxes,
//            so beware.
//
//            Experiment and have fun!
//**************************************************************************
 
#include <WINDOWS.H>
#include <stdio.H>
#include <stdlib.H>
#include <string.h>
#include <dos.h>
#include <memory.h>
 
#include "wrtrsamp.h"
#include "wrtr_rc.h"
#include "e32appc.h"              // Router APIs
//#include "ehndtw.h"               // Translation APIs

//DT header stuff
#define  EHNDT_SUCCESS              0
 
//*------------------------------------------------------------------------
//| Include PATTERNS.H which creates data patterns that are sent to and
//| received from the host.  Tell it to not declare the 32K patterns,
//| because that would require large model for this program.  We don't want
//| that because Windows would only allow one copy of the program to run at
//| a time.
//*------------------------------------------------------------------------
#define  NO_BIG_PATTERNS     1
#define  TAB '\x9'
#include "patterns.h"
 
 
//*------------------------------------------------------------------------
//| Global Variables
//*------------------------------------------------------------------------
static HANDLE  hInst;
static FARPROC lpfnGetClistDlg;
static BOOL    debug = FALSE;           /* Display debug messages?       */
char           pipdata[300] = {""};
unsigned char  pipout[300]  = {""};
BYTE           globalRTS;               /* Request to Send field         */
DeallocateEnum dealloctype;
SyncLevelEnum  SyncLevel;
BOOL           AllocateVerb;  /* True if ALLOCATE; False if EXT_ALLOCATE */
char           modename[10];
SecurityType   sectype;
char           userid[10];
char           password[10];
char           AsciiBuffer[300];       /* Buffer for the A2e dialog box  */
char           EbcdicBuffer[300];      /* Buffer for the E2a dialog box  */
int            globalret;
 
struct conv_list  /* Structure for the list of active conversation id's  */
  {
     unsigned char    convhandle[10];
     DWORD            convhex;
     BOOL             allocated;
  }  static convlistarray[MAX_CONVERSATIONS];
 
static DWORD conv_id;       /* Current conversation id      */
static DWORD convindex;     /* Current index to conv table  */
 
struct pattern  /* 16K structure for sending/receiving data */
  {
     unsigned char    llfield[LL_HDR];   /* LL header field */
     unsigned char    data[16384];       /* Data portion    */
  }  far *pattern1;
 
struct BigPattern     /* Define pointers to a 32K structure */
  {
     unsigned char    llfield[LL_HDR];
     unsigned char    data[32765];
  }  far *DataPtr, far *CustomPtr, far *RcvPtr;
 
HANDLE  hpattern1, hDataPtr, hTempRcv;
 
struct commandinfo  /* Structure for sending/receiving data */
  {
     unsigned char    llfield[LL_HDR];  /* LL header field  */
     unsigned char    data[64];
  }  command;
 
typedef struct tagREALMEM
  {
     WORD  wSegment;                  /* Segment for use by real-mode  */
     WORD  wSelector;                 /* Selector for protected-mode   */
     DWORD dwBytes;                   /* Number of bytes allocated     */
  }  REALMEM;
 
REALMEM  rmAllocated;       /* Block of DOS memory currently allocated */
 
typedef REALMEM far *LPRM;
 
typedef enum
  {
     EXTENDED = 0,
     REAL = 1
  }  MemoryEnum;
 
MemoryEnum Location;    /* Location for receive buffers */
 
struct     /* Structure for sending custom defined data */
  {
     MemoryEnum   MemLocation;
     long         TotalBytes;
     int          NumOfLLs;
     struct
       {
          long Size;      /* Size of LL               */
          char Start;     /* First character to send  */
       }  LLinfo[4];
  }  CustomLLs;
 
struct  /* Structure for receiving custom defined data */
  {
     MemoryEnum   MemLocation;
     long         NumOfLLs;
     long         Size;
  }  CustomRcv;
 
 
//*------------------------------------------------------------------------
//| Internal subroutines
//*------------------------------------------------------------------------
/*-------------------------------------------------------------------------*/
/* get_ll function -- Returns the LL as an integer                         */
/*                                                                         */
/*                    An LL field is byte-reversed when we receive it      */
/*                    from the host. This function reverses the bytes and  */
/*                    returns the integer value.                           */
/*                                                                         */
/*                    NOTE: get_ll and put_ll aren't actually used in this */
/*                    program for the sake of better showing how to        */
/*                    manipulate LL headers and data fields, but there's   */
/*                    no reason for you not to use them.                   */
/*-------------------------------------------------------------------------*/
int get_ll(LPSTR data)
{
  return(data[0] * 0x100 + data[1]);
}
 
/*-------------------------------------------------------------------------*/
/* put_ll function -- Stores an integer in byte-reversed format.           */
/*                                                                         */
/*                    An LL field must be byte-reversed when we send it    */
/*                    to the host. This function reverses the bytes of an  */
/*                    integer and stores them in the character array.      */
/*-------------------------------------------------------------------------*/
void put_ll(LPSTR data, int ll)
{
  data[0] = ll/0x100;
  data[1] = ll%0x100;
}
 
/*-------------------------------------------------------------------------*/
/* verify_pattern -- Initialize the pattern1 structure to 0's.             */
/*-------------------------------------------------------------------------*/
void init_pattern1()
{
  int i;
  pattern1->llfield[0] = '\0';
  pattern1->llfield[1] = '\0';
  for (i=0; i < sizeof(pattern1->data); i++)
    pattern1->data[i] = '\0';
}
 
/*-------------------------------------------------------------------------*/
/* verify_pattern -- Compare the data received from the host with what we  */
/*                   expect it to be.                                      */
/*-------------------------------------------------------------------------*/
int verify_pattern(unsigned int patternnum, unsigned int totallength)
{
  int i;
  LPSTR m, n;
 
  /* check out the data length; subtract the LL header */
  if ((totallength-LL_HDR) != pattern_length[patternnum])
    return BAD_LENGTH;
 
  /* next check the data field */
 
  /* 1st check for small patterns -- 0 thru 5 */
  if (patternnum < 6)
    {
      m = DataPtr->data;
      n = pattern_buffer[patternnum];
      for (i=0; i < pattern_length[patternnum]; i++)
        {
          if (m[i] != n[i])
            break;
        }
    }
  else if (patternnum == 6)  /* BIG Pattern. This is actually pattern 7. */
    {
      for(i=0,m=DataPtr->data; i < pattern_length[patternnum]; i++)
        {
          if ((i % 2) == 0)  /* i is an even number */
            {
              if (m[i] != (char) 0xAA)
                break;
            }
          else               /* i is an odd number  */
            {
              if (m[i] != (char) 0x55)
                break;
            }
        }
    }
  else  /* patternnum = 7, but this is actually pattern 8. */
    {
      for(i=0,m=DataPtr->data; i < pattern_length[patternnum]; i++)
        {
          if ((i % 2) == 0)  /* i is an even number */
            {
              if (m[i] != (char) 0x55)
                break;
            }
          else               /* i is an odd number  */
            {
              if (m[i] != (char) 0xAA)
                break;
            }
        }
    } /* pattern = 7 */
 
  if (i != pattern_length[patternnum])     /* error at position i+1 */
    return (i+1);
  else                                     /* pattern is okay       */
    return GOOD_DATA;
}
 
 
/************************************************************************/
/*                                                                      */
/* DO_Send_Error -- Process the SendError menu option.                  */
/*                                                                      */
/************************************************************************/
int PASCAL DO_Send_Error(HWND hWnd)
{
  int   nReturnCode;
  char  what;                         /* What received from server      */
  char  reqsToSend;                   /* Output Request to send field   */
  int   receivedLength;               /* Length of received data        */
  char  cBuffer[100];
  int   bytecount;
 
  lstrcpy((LPSTR)&command.data, (LPSTR)"Error");
  command.llfield[0] = 00;
  command.llfield[1] = lstrlen((LPSTR)&command.data) + LL_HDR;
  bytecount = sizeof(command.data);
  EHNDT_ASCIIToEBCDIC(hWnd,
                     (LPSTR)&command.data,
                     (LPSTR)&command.data,
                     lstrlen((LPSTR)&command.data),
                     &bytecount);
 
  nReturnCode = EHNAPPC_SendData(hWnd, conv_id,
                                 command.llfield[1],
                                 (LPVOID) &command,
                                 (LPBYTE) &globalRTS);
  if (nReturnCode != EHNAPPC_OK)  /* Bad return code?             */
    {
      wsprintf(cBuffer,"Send data failed with Return Code = %d, %s.",
               nReturnCode, (LPSTR)returnstring[nReturnCode]);
      MessageBox(GetFocus(), cBuffer, "ERROR", MB_ICONSTOP);
      return(nReturnCode);        /* Yes, return the error        */
    }
 
  nReturnCode = EHNAPPC_Flush(hWnd, conv_id);
  if (debug)
    {
      wsprintf(cBuffer, "Flush Return Code = %d, %s.",
               nReturnCode, (LPSTR)returnstring[nReturnCode]);
      MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
    }
 
  do {
    nReturnCode = EHNAPPC_SendData(hWnd, conv_id,
                                   0,
                                   (LPVOID) &command,
                                   (LPBYTE) &globalRTS);
  } while ((globalRTS == 0) && (nReturnCode == EHNAPPC_OK));
 
  if (debug)
    {
      wsprintf(cBuffer,"globalRTS = %d, Send Data RC = %d, %s.",
               globalRTS, nReturnCode, (LPSTR)returnstring[nReturnCode]);
      MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
    }
 
  /* Receive the "Error" string back from the host   */
  nReturnCode = EHNAPPC_ReceiveAndWait(hWnd,conv_id,
                                       EHNAPPC_LL,
                                       sizeof(command),
                                       (LPVOID) &command,
                                       (LPBYTE) &what,
                                       (LPBYTE) &reqsToSend,
                                       (LPWORD) &receivedLength);
  if (nReturnCode != EHNAPPC_OK)
    {
      wsprintf(cBuffer,"Rcv And Wait RC = %d, %s.",
               nReturnCode, (LPSTR)returnstring[nReturnCode]);
      MessageBox(GetFocus(), cBuffer, "ERROR", MB_ICONSTOP);
      return (nReturnCode);
    }
 
  bytecount = sizeof(command.data);
  EHNDT_EBCDICToASCII(hWnd,
                      (LPSTR) command.data,
                      (LPSTR) command.data,
                      receivedLength - LL_HDR,
                      &bytecount);
 
  if (0 != memcmp((LPSTR) command.data, "Error", bytecount))
    {
      MessageBox(GetFocus(), "Host did not send 'Error' string correctly!",
                 "ERROR", MB_OK);
    }
 
  nReturnCode = EHNAPPC_SendError(hWnd, conv_id, (LPBYTE)&reqsToSend);
  if (nReturnCode != EHNAPPC_OK)
     return(nReturnCode);
 
  init_pattern1();
 
  /* turn line around  -- router will flush */
  nReturnCode = EHNAPPC_PrepareToReceive(hWnd,conv_id);
 
  do {
    nReturnCode = EHNAPPC_ReceiveImmediate(hWnd,conv_id,
                                           EHNAPPC_LL,
                                           256,
                                           (LPVOID) pattern1,
                                           (LPBYTE) &what,
                                           (LPBYTE) &reqsToSend,
                                           (LPWORD) &receivedLength);
  } while(nReturnCode == EHNAPPC_UNSUCCESSFUL);
 
  if (nReturnCode != EHNAPPC_OK)  /* if bad return code           */
    {
      wsprintf(cBuffer, "RCV IMMED failed with RC = %d, %s.",
               nReturnCode, (LPSTR)returnstring[nReturnCode]);
      MessageBox(GetFocus(), cBuffer, "ERROR", MB_ICONSTOP);
    }
 
  return(nReturnCode);
}
 
 
/************************************************************************/
/*                                                                      */
/* DO_Request_Send -- Process the Request To Send menu option.          */
/*                                                                      */
/************************************************************************/
int PASCAL DO_Request_Send(HWND hWnd)
{
  int      nReturnCode;
  char     what;                      /* What received from server      */
  char     reqsToSend;                /* Output Request to send field   */
  int      receivedLength;            /* Length of received data        */
  char     cBuffer[100];
  char     tempchar;
  char     count;
  int      bytecount;
 
  lstrcpy((LPSTR)&command.data, (LPSTR)"RequestToSend");
  command.llfield[0] = 00;
  command.llfield[1] = lstrlen((LPSTR)&command.data) + LL_HDR;
  bytecount = sizeof(command.data);
  EHNDT_ASCIIToEBCDIC(hWnd,
                     (LPSTR) command.data,
                     (LPSTR) command.data,
                     lstrlen((LPSTR)&command.data),
                     &bytecount);
 
  /* Tell the host what option we want to execute (RequestToSend) */
  nReturnCode = EHNAPPC_SendData(hWnd, conv_id,
                                 command.llfield[1],
                                 (LPVOID) &command,
                                 (LPBYTE) &globalRTS);
  if (nReturnCode != EHNAPPC_OK)  /* Bad return code?             */
    {
      wsprintf(cBuffer,"Send data failed with Return Code = %d, %s.",
               nReturnCode, (LPSTR)returnstring[nReturnCode]);
      MessageBox(GetFocus(), cBuffer, "ERROR", MB_ICONSTOP);
      return(nReturnCode);        /* Yes, return the error        */
    }
 
  nReturnCode = EHNAPPC_Flush(hWnd,conv_id);
  if (nReturnCode != EHNAPPC_OK)  /* Bad return code?             */
    {
      wsprintf(cBuffer,"FLUSH Return Code = %d, %s",
               nReturnCode, (LPSTR)returnstring[nReturnCode]);
      MessageBox(GetFocus(), cBuffer, "ERROR", MB_ICONSTOP);
      return(nReturnCode);        /* Yes, return the error        */
    }
 
 
  /* Now send the host many, many copies of the alphabet.  It     */
  /* will sit in a loop and receive ~50 copies before it sends a  */
  /* request to send.                                             */
  lstrcpy((LPSTR)&pattern1->data, (LPSTR)"abcdefghijklmnopqrstuvwxyz");
  pattern1->llfield[0] = 0;
  pattern1->llfield[1] = 29;    /* 26 letters + null + LL-field   */
  bytecount = sizeof(pattern1->data);
  EHNDT_ASCIIToEBCDIC(hWnd,
                     (LPSTR) pattern1->data,
                     (LPSTR) pattern1->data,
                     26,
                     &bytecount);
  pattern1->data[26] = '\0';    /* null terminate for the host tp */
 
  do {
    nReturnCode = EHNAPPC_SendData(hWnd, conv_id,
                                   pattern1->llfield[1],
                                   (LPVOID) pattern1,
                                   (LPBYTE) &globalRTS);
    nReturnCode = EHNAPPC_Flush(hWnd, conv_id);
  } while ((globalRTS == 0) && (nReturnCode == EHNAPPC_OK));
 
  if (debug)
    {
      wsprintf(cBuffer, "SEND globalRTS = %d; FLUSH RC = %d, %s",
               globalRTS, nReturnCode, (LPSTR)returnstring[nReturnCode]);
      MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
    }
 
  /* turn line around  -- router will flush */
  reqsToSend = 0;
  init_pattern1();
  nReturnCode = EHNAPPC_PrepareToReceive(hWnd,conv_id);
  if (debug)
    {
      wsprintf(cBuffer,"PREP TO RCV Return Code = %d, %s",
               nReturnCode, (LPSTR)returnstring[nReturnCode]);
      MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
    }
 
  /* Receive data for a while and then interrupt the host with    */
  /* a Request To Send.                                           */
  for (count = 0; count < 10; ++count)
    {
      nReturnCode = EHNAPPC_ReceiveAndWait(hWnd, conv_id,
                                           EHNAPPC_BUFFER,
                                           256,
                                           (LPVOID) pattern1,
                                           (LPBYTE) &what,
                                           (LPBYTE) &reqsToSend,
                                           (LPWORD) &receivedLength);
      if (nReturnCode != EHNAPPC_OK)
        {
          wsprintf(cBuffer,"RCV and Wait failed with RC = %d, %s",
                   nReturnCode, (LPSTR)returnstring[nReturnCode]);
          MessageBox(GetFocus(), cBuffer, "ERROR", MB_ICONSTOP);
          return(nReturnCode);         /*  Return the error  */
        }
    }
 
  if (debug)
    {
      wsprintf(cBuffer, "RCV and Wait RC = %d, %s",
               nReturnCode, (LPSTR)returnstring[nReturnCode]);
      MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
 
      wsprintf(cBuffer, "What Received = %d", what);
      MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
 
      wsprintf(cBuffer, "RTS = %d", reqsToSend);
      MessageBox(GetFocus(), cBuffer, "RTS FLAG", MB_OK);
    }
 
  /*-----------------------------------------------------*/
  /* Now interrupt the host by issuing a Request To Send */
  /*-----------------------------------------------------*/
  if (debug)
    {
      MessageBox(GetFocus(), "Now Issue RTS", "INFO", MB_OK);
    }
  nReturnCode = EHNAPPC_RqsToSend(hWnd, conv_id);
 
  if (nReturnCode != EHNAPPC_OK)  /* if bad return code           */
    return(nReturnCode);          /*  Yes, return the error       */
  else if (debug)
    {
      wsprintf(cBuffer, "REQUEST TO SEND RC = %d, %s",
               nReturnCode, (LPSTR)returnstring[nReturnCode]);
      MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
    }
 
  /* wait for Send Direction*/
  do {
    nReturnCode = EHNAPPC_ReceiveAndWait(hWnd,conv_id,
                                         EHNAPPC_BUFFER,
                                         256,
                                         (LPVOID) pattern1,
                                         (LPBYTE) &what,
                                         (LPBYTE) &reqsToSend,
                                         (LPWORD) &receivedLength);
    if (nReturnCode != EHNAPPC_OK)
      {
        wsprintf(cBuffer,"Failed to get direction with RCV-Wait RC = %d, %s",
                 nReturnCode, (LPSTR)returnstring[nReturnCode]);
        MessageBox(GetFocus(), cBuffer, "ERROR", MB_ICONSTOP);
        break;
      }
  } while (what != EHNAPPC_RECEIVEDSEND);
 
  return(nReturnCode);
}
 
/************************************************************************/
/*                                                                      */
/* DO_Flush -- Process the Flush menu option.                           */
/*                                                                      */
/************************************************************************/
int PASCAL DO_Flush(HWND hWnd)
{
  int      nReturnCode;
  char     what;                      /* What received from server      */
  char     reqsToSend;                /* Output Request to send field   */
  int      receivedLength;            /* Length of received data        */
  char     cBuffer[100];
  int      bytecount;
 
  lstrcpy((LPSTR)&command.data, (LPSTR)"Flush");
  command.llfield[0] = 00;
  command.llfield[1] = lstrlen((LPSTR)&command.data) + LL_HDR;
  bytecount = sizeof(command.data);
  EHNDT_ASCIIToEBCDIC(hWnd,
                     (LPSTR) command.data,
                     (LPSTR) command.data,
                     lstrlen((LPSTR)&command.data),
                     &bytecount);
 
  nReturnCode = EHNAPPC_SendData(hWnd, conv_id,
                                 command.llfield[1],
                                (LPVOID) &command,
                                (LPBYTE) &globalRTS);
 
  nReturnCode = EHNAPPC_Flush(hWnd,conv_id);
  if (nReturnCode != EHNAPPC_OK)  /* Bad return code?       */
    {
      return(nReturnCode);        /* Yes, return the error  */
    }
 
  globalRTS = 0;
  do {
    nReturnCode = EHNAPPC_SendData(hWnd, conv_id,
                                   0,
                                   (LPVOID) &command,
                                   (LPBYTE) &globalRTS);
  } while ((globalRTS == 0) && (nReturnCode == EHNAPPC_OK));
 
  if (debug)
    {
      wsprintf(cBuffer,"globalRTS = %d, Return Code = %d, %s",
               globalRTS, nReturnCode, (LPSTR)returnstring[nReturnCode]);
      MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
    }
 
  init_pattern1();
 
  /* turn line around  -- router will flush */
  nReturnCode = EHNAPPC_PrepareToReceive(hWnd,conv_id);
 
  do {
    nReturnCode = EHNAPPC_ReceiveImmediate(hWnd, conv_id,
                                           EHNAPPC_LL,
                                           256,
                                           (LPVOID) pattern1,
                                           (LPBYTE) &what,
                                           (LPBYTE) &reqsToSend,
                                           (LPWORD) &receivedLength);
  } while(nReturnCode == EHNAPPC_UNSUCCESSFUL);
 
  if (debug)
    {
      wsprintf(cBuffer,"What Received = %d", what);
      MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
 
      wsprintf(cBuffer,"RCV IMMED RC = %d, %s",
               nReturnCode, (LPSTR)returnstring[nReturnCode]);
      MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
    }
 
  if (nReturnCode != EHNAPPC_OK)  /* if bad return code      */
    return(nReturnCode);          /*  Yes, return the error  */
 
  bytecount = sizeof(pattern1->data);
  EHNDT_EBCDICToASCII(hWnd,
                      (LPSTR) pattern1->data,
                      (LPSTR) pattern1->data,
                      receivedLength - LL_HDR,
                      &bytecount);
 
  if (0 != memcmp((LPSTR) pattern1->data, "Flushed", bytecount))
    {
      MessageBox(GetFocus(),
                 "Host did not send 'Flushed' string correctly!",
                 "ERROR", MB_OK);
    }
 
  /* Must issue second receive to get Send indicator. */
  do {
    nReturnCode = EHNAPPC_ReceiveImmediate(hWnd, conv_id,
                                           EHNAPPC_LL,
                                           0,
                                           (LPVOID) &command,
                                           (LPBYTE) &what,
                                           (LPBYTE) &reqsToSend,
                                           (LPWORD) &receivedLength);
  } while (nReturnCode == EHNAPPC_UNSUCCESSFUL);
 
  if (nReturnCode != EHNAPPC_OK)
    {
      wsprintf(cBuffer,"RCV IMMED RC = %d, %s",
               nReturnCode, (LPSTR)returnstring[nReturnCode]);
      MessageBox(GetFocus(), cBuffer, "ERROR", MB_OK);
    }
 
  return(nReturnCode);
}
 
/************************************************************************/
/*                                                                      */
/* DO_Prepare_To_Receive -- Process the Prepare to receive menu option. */
/*                                                                      */
/************************************************************************/
int PASCAL DO_Prepare_To_Receive(HWND hWnd)
{
  int      nReturnCode;
  char     what;                      /* What received from server      */
  char     reqsToSend;                /* Output Request to send field   */
  int      receivedLength;            /* Length of received data        */
  char     cBuffer[100];
  int      bytecount;
 
  lstrcpy((LPSTR)&command.data, (LPSTR)"PrepareToReceive");
  command.llfield[0] = 00;
  command.llfield[1] = lstrlen((LPSTR)&command.data) + LL_HDR;
  bytecount = sizeof(command.data);
  EHNDT_ASCIIToEBCDIC(hWnd,
                      command.data,
                      command.data,
                      lstrlen((LPSTR)&command.data),
                      &bytecount);
 
  nReturnCode = EHNAPPC_SendData(hWnd, conv_id,
                                 command.llfield[1],
                                 (LPVOID) &command,
                                 (LPBYTE) &globalRTS);
  if (nReturnCode != EHNAPPC_OK)  /* Bad return code?      */
    {
      wsprintf(cBuffer, "Send Data RC = %d, %s",
               nReturnCode, (LPSTR)returnstring[nReturnCode]);
      MessageBox(GetFocus(), cBuffer, "ERROR", MB_ICONSTOP);
      return(nReturnCode);        /* Yes, return the error */
    }
 
  init_pattern1();
 
  /* turn line around  -- router will flush */
  nReturnCode = EHNAPPC_PrepareToReceive(hWnd,conv_id);
  if (nReturnCode != EHNAPPC_OK)
    return (nReturnCode);
 
  do {
    nReturnCode = EHNAPPC_ReceiveImmediate(hWnd, conv_id,
                                           EHNAPPC_LL,
                                           256,
                                           (LPVOID) pattern1,
                                           (LPBYTE) &what,
                                           (LPBYTE) &reqsToSend,
                                           (LPWORD) &receivedLength);
  } while (nReturnCode == EHNAPPC_UNSUCCESSFUL);
 
  if (debug)
    {
      wsprintf(cBuffer, "What Received = %d", what);
      MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
 
      wsprintf(cBuffer,"RCV IMMED RC = %d, %s",
               nReturnCode, (LPSTR)returnstring[nReturnCode]);
      MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
    }
 
  if (nReturnCode != EHNAPPC_OK)  /* if bad return code           */
    return(nReturnCode);          /*  Yes, return the error       */
 
  bytecount = sizeof(pattern1->data);
  EHNDT_EBCDICToASCII(hWnd,
                     (LPSTR) pattern1->data,
                     (LPSTR) pattern1->data,
                     receivedLength - LL_HDR,
                     &bytecount);
 
 if (0 != memcmp((LPSTR)&pattern1->data, "PrepareToReceive", bytecount))
   {
     MessageBox(GetFocus(),
                "Host did not send 'PrepareToReceive' string correctly!",
                "ERROR", MB_OK);
   }
 
  /* Must issue second receive to get Send indicator.              */
  do {
    nReturnCode = EHNAPPC_ReceiveImmediate(hWnd, conv_id,
                                           EHNAPPC_LL,
                                           256,
                                           (LPVOID) pattern1,
                                           (LPBYTE) &what,
                                           (LPBYTE) &reqsToSend,
                                           (LPWORD) &receivedLength);
  } while (nReturnCode == EHNAPPC_UNSUCCESSFUL);
 
  if (nReturnCode != EHNAPPC_OK)
    {
      wsprintf(cBuffer,"RCV IMMED RC = %d, %s",
               nReturnCode, (LPSTR)returnstring[nReturnCode]);
      MessageBox(GetFocus(), cBuffer, "ERROR", MB_OK);
    }
 
  return(nReturnCode);
}
 
 
/************************************************************************/
/*                                                                      */
/* DO_Flip -- Process the Flip menu option.                             */
/*                                                                      */
/* The purpose of the flip option is to allow a conversation to be put  */
/* in RECEIVE state, so STATE CHECK return codes can be received. Use   */
/* the restore option to use this conversation further.                 */
/*                                                                      */
/************************************************************************/
int PASCAL DO_Flip(HWND hWnd)
{
  int      nReturnCode;
  char     what;                      /* What received from server      */
  char     reqsToSend;                /* Output Request to send field   */
  int      receivedLength;            /* Length of received data        */
  char     cBuffer[100];
  int      bytecount;
 
  lstrcpy((LPSTR)&command.data, (LPSTR)"Flip");
  command.llfield[0] = 00;
  command.llfield[1] = lstrlen((LPSTR)&command.data) + LL_HDR;
  bytecount = sizeof(command.data);
  EHNDT_ASCIIToEBCDIC(hWnd,
                      command.data,
                      command.data,
                      lstrlen((LPSTR)&command.data),
                      &bytecount);
 
  nReturnCode = EHNAPPC_SendData(hWnd, conv_id,
                                 command.llfield[1],
                                 (LPVOID) &command,
                                 (LPBYTE) &globalRTS);
 
  /* turn line around  -- router will flush */
  nReturnCode = EHNAPPC_PrepareToReceive(hWnd,conv_id);
 
  return(nReturnCode);
}
 
 
/************************************************************************/
/*                                                                      */
/* DO_Restore -- Process the Restore menu option.                       */
/*                                                                      */
/* This option is used to put a conversation back into SEND state after */
/* it has been flipped with the flip option.                            */
/*                                                                      */
/************************************************************************/
int PASCAL DO_Restore(HWND hWnd)
{
  int      nReturnCode;
  char     what;                      /* What received from server      */
  char     reqsToSend;                /* Output Request to send field   */
  int      receivedLength;            /* Length of received data        */
  char     cBuffer[100];
 
  nReturnCode = EHNAPPC_RqsToSend(hWnd, conv_id);
  if (nReturnCode != EHNAPPC_OK)
    return nReturnCode;
 
  /* wait for direction */
  do {
    nReturnCode = EHNAPPC_ReceiveAndWait(hWnd, conv_id,
                                         EHNAPPC_BUFFER,
                                         sizeof(pattern1),
                                         (LPVOID) pattern1,
                                         (LPBYTE) &what,
                                         (LPBYTE) &reqsToSend,
                                         (LPWORD) &receivedLength);
    if (nReturnCode != EHNAPPC_OK)
      {
        wsprintf(cBuffer,"Failed to get direction with RCV-Wait RC = %d, %s",
                 nReturnCode, (LPSTR)returnstring[nReturnCode]);
        MessageBox(GetFocus(), cBuffer, "ERROR", MB_ICONSTOP);
        return(nReturnCode);
      }
  } while (what != EHNAPPC_RECEIVEDSEND);
 
  if (debug)
    {
      wsprintf(cBuffer,"What Received = %d, Rcv and Wait RC = %d, %s",
               what, nReturnCode, (LPSTR)returnstring[nReturnCode]);
      MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
    }
 
  return(nReturnCode);
}
 
 
/************************************************************************/
/*                                                                      */
/* DO_Confirm -- Process the Confirm menu option.                       */
/*                                                                      */
/************************************************************************/
int PASCAL DO_Confirm(HWND hWnd)
{
  char reqsToSend;                /* Request to send field   */
 
  return(EHNAPPC_Confirm(hWnd, conv_id, (LPBYTE) &reqsToSend));
}
 
 
/************************************************************************/
/*                                                                      */
/* DO_Confirmed -- Process the Confirmed menu option.                   */
/*                                                                      */
/************************************************************************/
int PASCAL DO_Confirmed(HWND hWnd)
{
  int      nReturnCode;
  char     what;                      /* What received from server      */
  char     reqsToSend;                /* Output Request to send field   */
  int      receivedLength;            /* Length of received data        */
  char     cBuffer[100];
  int      bytecount;
  char     luname[EHNAPPC_SYSNAME_LENGTH];
  char     hostname[EHNAPPC_SYSNAME_LENGTH];
 
  /* Make sure the sync level of the conversation is CONFIRM */
  nReturnCode = EHNAPPC_GetAttributes(hWnd, conv_id,
                                     (LPBYTE) &SyncLevel,
                                     (LPSTR) &modename,
                                     (LPSTR) &luname,
                                     (LPSTR) &hostname,
                                     (LPSTR) &userid);
  if (nReturnCode != EHNAPPC_OK)  /* Bad return code?             */
    {
      wsprintf(cBuffer, "Get Attributes RC = %d, %s",
               nReturnCode, (LPSTR)returnstring[nReturnCode]);
      MessageBox(GetFocus(), cBuffer, "ERROR", MB_ICONSTOP);
      return(nReturnCode);        /*  Yes, return the error       */
    }
 
  if (SyncLevel == EHNAPPC_SYNCLEVELNONE)
    {
      /* Make sure EHNAPPC gives correct return code */
      return(EHNAPPC_Confirmed(hWnd, conv_id));
    }
 
  lstrcpy((LPSTR)&command.data, (LPSTR)"Confirm");
  command.llfield[0] = 00;
  command.llfield[1] = lstrlen((LPSTR)&command.data) + LL_HDR;
  bytecount = sizeof(command.data);
  EHNDT_ASCIIToEBCDIC(hWnd,
                      command.data,
                      command.data,
                      lstrlen((LPSTR)&command.data),
                      &bytecount);
 
  /* Tell host what we're going to do */
  nReturnCode = EHNAPPC_SendData(hWnd, conv_id,
                                 command.llfield[1],
                                 (LPVOID) &command,
                                 (LPBYTE) &globalRTS);
 
  /* turn line around  -- router will flush */
  nReturnCode = EHNAPPC_PrepareToReceive(hWnd, conv_id);
 
  do {
    nReturnCode = EHNAPPC_ReceiveImmediate(hWnd, conv_id,
                                           EHNAPPC_LL,
                                           sizeof(command),
                                           (LPVOID) &command,
                                           (LPBYTE) &what,
                                           (LPBYTE) &reqsToSend,
                                           (LPWORD) &receivedLength);
  } while(nReturnCode == EHNAPPC_UNSUCCESSFUL);
 
  if (nReturnCode != EHNAPPC_OK)  /* Bad return code?             */
    {
      wsprintf(cBuffer,"RCV IMMED RC = %d, %s",
               nReturnCode, (LPSTR)returnstring[nReturnCode]);
      MessageBox(GetFocus(), cBuffer, "ERROR", MB_ICONSTOP);
      return(nReturnCode);        /*  Yes, return the error       */
    }
 
  if (debug)
    {
      wsprintf(cBuffer,"What Received = %d", what);
      MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
 
      wsprintf(cBuffer,"RCV IMMED RC = %d, %s",
               nReturnCode, (LPSTR)returnstring[nReturnCode]);
      MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
    }
 
 
  nReturnCode = EHNAPPC_Confirmed(hWnd, conv_id);
  if (nReturnCode != EHNAPPC_OK)  /* Bad return code?             */
    {
      wsprintf(cBuffer, "Confirmed RC = %d, %s",
               nReturnCode, (LPSTR)returnstring[nReturnCode]);
      MessageBox(GetFocus(), cBuffer, "ERROR", MB_ICONSTOP);
      return(nReturnCode);
    }
 
 
  /* Must issue second receive to get Send indicator.             */
  do {
    nReturnCode = EHNAPPC_ReceiveImmediate(hWnd, conv_id,
                                           EHNAPPC_LL,
                                           sizeof(command),
                                           (LPVOID) &command,
                                           (LPBYTE) &what,
                                           (LPBYTE) &reqsToSend,
                                           (LPWORD) &receivedLength);
  } while(nReturnCode == EHNAPPC_UNSUCCESSFUL);
 
  if (nReturnCode != EHNAPPC_OK)  /* Bad return code?             */
    {
      wsprintf(cBuffer,"RCV IMMED RC = %d, %s",
               nReturnCode, (LPSTR)returnstring[nReturnCode]);
      MessageBox(GetFocus(), cBuffer, "ERROR", MB_ICONSTOP);
    }
 
  return(nReturnCode);
}
 
 
/************************************************************************/
/*                                                                      */
/* SetCONID -- Prompt the user to select a conversation ID.             */
/*                                                                      */
/************************************************************************/
BOOL SetCONID (HWND hWnd)
{
  char  cBuffer[41];
  int   rc;
 
  rc = DialogBox(hInst, "DB_LISTCONV", hWnd, lpfnGetClistDlg);
  if (debug)
    {
      wsprintf(cBuffer, "CONV ID = %10lx", conv_id);
      MessageBox(GetFocus(), cBuffer, "Conv Info!", MB_OK);
    }
  return rc;
}
 
 
/************************************************************************/
/* GlobalDosLargest -- Allocate the maximum amount of conventional mem. */
/*                                                                      */
/*  IMPLEMENTATION NOTES:                                               */
/*  The Windows function, GlobalDosAlloc, does not return the amount    */
/*  of available memory, and there appears to be no other way to        */
/*  determine how much free DOS memory there is.  Since Windows claims  */
/*  all DOS memory when it loads, the DPMI interface will not work.     */
/*                                                                      */
/*  This is a binary algorithm to determine the true amount of memory   */
/*  available.  To speed things up, first the requested amount is       */
/*  allocated.  If that does not work, then try to allocate the         */
/*  minimum amount.  If that works then try to successively increase    */
/*  the requested amount.                                               */
/************************************************************************/
BOOL FAR PASCAL GlobalDosLargest(DWORD dwMaximumAmount,
                                 DWORD dwMinimumAmount,
                                 LPRM  lpRM)
{
  DWORD dwRequestAmount;
  DWORD dwSuccessAmount;
  DWORD dwFailureAmount;
  DWORD dwPtr;
/* 
  // try allocating the maximum amount needed
  if ( (dwPtr = GlobalDosAlloc(dwMaximumAmount)) != NULL )
  
    {
      lpRM->dwBytes = dwMaximumAmount; // It worked!  We are done!
    }
  else
    {
      // That didn't work, try allocating the minimum amount
      dwFailureAmount = dwMaximumAmount;
      if ( (dwPtr = GlobalDosAlloc(dwMinimumAmount)) == NULL )
	  
        {
          lpRM->dwBytes = 0; // It failed. We are done.
        }
      else
        {
          // There is at least the minimum available, find the exact amount
          dwSuccessAmount = dwMinimumAmount;
          GlobalDosFree(LOWORD(dwPtr));
 
          while ((dwSuccessAmount+1) < dwFailureAmount)
            {
              dwRequestAmount = (dwFailureAmount + dwSuccessAmount) / 2;
              if ((dwPtr = GlobalDosAlloc(dwRequestAmount)) != NULL)
                {
                  dwSuccessAmount = dwRequestAmount;
                  GlobalDosFree(LOWORD(dwPtr));  // free it temporarily
                }
              else
                {
                  dwFailureAmount = dwRequestAmount;
                }
            }
          // Now we know what the largest available memory is.  It is
          // not always the last RequestAmount, but it IS always the
          // last SuccessAmount.
          if ((dwPtr = GlobalDosAlloc(dwSuccessAmount)) != NULL)
            {
              lpRM->dwBytes = dwSuccessAmount;
            }
        }
    } 
  lpRM->wSelector = LOWORD(dwPtr);
  lpRM->wSegment  = HIWORD(dwPtr);
  return (dwPtr != NULL); 
  */ 
  //dxd  Disabled DOS allocate for win95 so this shouldn't be called
  MessageBox(GetFocus(),
                           "Tried to do GlobalDOS memory function.",
                           "UNEXPECTED MEMORY ALLOCATION CALL", MB_ICONSTOP);
  
  return (NULL);

}
 
 
//*------------------------------------------------------------------------
//| WinMain:
//|     Parameters:
//|         hInstance     - Handle to current Data Segment
//|         hPrevInstance - Handle to previous Data Segment (NULL if none)
//|         lpszCmdLine   - Long pointer to command line info
//|         CmdShow       - Integer value specifying how to start app.,
//|                            (Iconic or Normal)
//*------------------------------------------------------------------------
int PASCAL WinMain (HANDLE hInstance,
                    HANDLE hPrevInstance,
                    LPSTR  lpszCmdLine,
                    int    nCmdShow)
{
  int nReturn;
 
  if (Init(hInstance, hPrevInstance,lpszCmdLine,nCmdShow))
    {
      nReturn = DoMain(hInstance);
      CleanUp();
    }
  return nReturn;
}
 
//*------------------------------------------------------------------------
//| Init
//|     Initialization for the program is done here:
//|     1)  Register the window class (if this is the first instance)
//|     2)  Create the desktop window for the app.
//|     3)  Allocate the conversation
//|     4)  Show the desktop window in the manner requested by the User.
//|
//*------------------------------------------------------------------------
BOOL Init(HANDLE hInstance,   HANDLE hPrevInstance,
          LPSTR  lpszCmdLine, int    nCmdShow)
{
  WNDCLASS    rClass;
  HWND        hWnd;
  int         i;
  char        cBuffer[100];
  DWORD       dwFree;
 
  hInst = hInstance;
 
  if (!hPrevInstance)
    {
      /*  Register Class for First Overlapped Window  */
      rClass.lpszClassName = "DLLTester";
      rClass.hInstance     = hInst;
      rClass.lpfnWndProc   = OverlappedWindowProc1;
      rClass.hCursor       = 0;
      rClass.hIcon         = LoadIcon(hInstance, "BASEICON");
      rClass.lpszMenuName  = "BASEMENU2";
      rClass.hbrBackground = COLOR_WINDOW+1;
      rClass.style         = 0;
      rClass.cbClsExtra    = 0;
      rClass.cbWndExtra    = 0;
 
      if (!RegisterClass( &rClass))
          return FALSE;
    }
 
  hWnd = CreateWindow("DLLTester",
                      "Router Tester",
                      WS_OVERLAPPEDWINDOW,
                      CW_USEDEFAULT,
                      CW_USEDEFAULT,
                      CW_USEDEFAULT,
                      CW_USEDEFAULT,
                      NULL,
                      NULL,
                      hInstance,
                      NULL);
 
  ShowWindow(hWnd,nCmdShow);
  init_patterns();
 
  /*------------------------------------------------------------------*/
  /* Make sure there's enough memory to allocate 16K and 32K chunks.  */
  /* If there's not, tell the user how much is available.             */
  /*------------------------------------------------------------------*/
  //win95 always returns 0 so removed
  //dwFree = GlobalCompact(0);
  // if (dwFree < 49154)
  //  {
  //    wsprintf(cBuffer, "Largest block of free contiguous memory = %lu bytes",
  //             dwFree);
  //    MessageBox(GetFocus(),cBuffer, "GlobalCompact result", MB_OK);
  //  }
 
  hpattern1 = GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT, 16386);
  pattern1  = (struct pattern  far *) GlobalLock(hpattern1);
  if ((hpattern1 == 0) || (pattern1 == NULL))
    {
      MessageBox(GetFocus(), "Failed to allocate or lock pattern1!",
                 "TEST DRIVER MEMORY ALLOCATION ERROR", MB_ICONSTOP);
      MessageBox(GetFocus(), "Exit right now or you will hang or GP fault!",
                 "TEST DRIVER MEMORY ALLOCATION ERROR", MB_ICONSTOP);
    }
 
  hDataPtr = GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT, 32767);
  DataPtr = (struct BigPattern  far *) GlobalLock(hDataPtr);
  if ((hDataPtr == 0) || (DataPtr == NULL))
    {
      MessageBox(GetFocus(), "Failed to allocate or lock DataPtr!",
                 "TEST DRIVER MEMORY ALLOCATION ERROR", MB_ICONSTOP);
      MessageBox(GetFocus(), "Exit right now or you will hang or GP fault!",
                 "TEST DRIVER MEMORY ALLOCATION ERROR", MB_ICONSTOP);
    }
 
  /* To store these values, the array has to be changed to far pointers. */
  /* REMEMBER: the host driver shares PATTERNS.H with this driver.       */
// pattern_buffer[6] = (LPSTR) DataPtr; /* Store these ptrs for verification */
// pattern_buffer[7] = (LPSTR) DataPtr;
 
  for (i=0; i < (EHNAPPC_MAX_SYSTEMS-1); i++ )
    {
      convlistarray[i].allocated = FALSE;
    }
 
  return hWnd;
}
 
//*------------------------------------------------------------------------
//| DoMain:
//|     This is the main loop for the application:
//*------------------------------------------------------------------------
int  DoMain(HANDLE hInstance)
{
  MSG msg;
 
  while (GetMessage(&msg,NULL,0,0))
    {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }
  return msg.wParam;
}
 
//*------------------------------------------------------------------------
//| CleanUp:
//|     Any last-minute application cleanup activities are done here:
//*------------------------------------------------------------------------
void CleanUp(void)
{
}
 
//*------------------------------------------------------------------------
//| OverlappedWindowProc1
//|     Parameters:
//|         hWnd    - Handle to Window which message is delivered to.
//|         msgID   - ID number of message
//|         wParam  - 16-bit parameter
//|         lParam  - 32-bit parameter
//|
//*------------------------------------------------------------------------
long FAR PASCAL OverlappedWindowProc1 (HWND     hWnd,
                                       WORD     wMsgID,
                                       WORD     wParam,
                                       LONG     lParam)
{
  static FARPROC lpfnAllocateInfoDlg;
  static FARPROC lpfnExtAllocateInfoDlg;
  static FARPROC lpfnDeallocateDlg;
  static FARPROC lpfnSystemList;
  static FARPROC lpfnFullSystemList;
  static FARPROC lpfnGetUserIDDlg;
  static FARPROC lpfnTakeMemDlg;
  static FARPROC lpfnCustomDataDlg;
  static FARPROC lpfnCustomRcvDlg;
  static FARPROC lpfnA2eDlg;
  static FARPROC lpfnLocationDlg;
  static FARPROC lpfnAboutDlg;
 
  struct                        /* Structure to query router capabilities */
    {
      struct appcrtrcap_hdr   hdr;
      struct appcrtrcap_mult  mult;          /* Optimal multiplier        */
      struct appcrtrcap_query qryconv;       /* Conv. state service verb  */
      struct appcrtrcap_query qryext;        /* Extended allocate verb    */
      struct appcrtrcap_query qryfullsys;    /* Query full sys list verb  */
    } caplist;
 
  char     cBuffer[100];
  char     cMsg[200];
  char     luname[EHNAPPC_SYSNAME_LENGTH];
  char     hostname[EHNAPPC_SYSNAME_LENGTH];
  int      nReturnCode;
  int      i, j, DataByte;
  char     what;                        /* What received from server      */
  char     reqsToSend;                  /* Request to send field          */
  int      receivedLength;              /* Length of received data        */
  int      patternnum;
  int      TotalLength, MaxLength;
  REALMEM  rmTemp;
  int      bytecount;
  LPSTR    m, n, tempptr;
 
  switch (wMsgID)
    {
    case WM_CREATE:
      lpfnAllocateInfoDlg    = MakeProcInstance(AllocateInfoDlgProc,hInst);
      lpfnExtAllocateInfoDlg = MakeProcInstance(ExtAllocateInfoDlgProc,hInst);
      lpfnDeallocateDlg      = MakeProcInstance(DeallocateDlgProc,hInst);
      lpfnSystemList         = MakeProcInstance(SystemListProc,hInst);
      lpfnFullSystemList     = MakeProcInstance(FullSystemListProc,hInst);
      lpfnGetUserIDDlg       = MakeProcInstance(GetUserIDDlgProc,hInst);
      lpfnTakeMemDlg         = MakeProcInstance(TakeMemDlgProc,hInst);
      lpfnCustomDataDlg      = MakeProcInstance(CustomDataDlgProc,hInst);
      lpfnCustomRcvDlg       = MakeProcInstance(CustomRcvDlgProc,hInst);
      lpfnA2eDlg             = MakeProcInstance(A2eDlgProc,hInst);
      lpfnLocationDlg        = MakeProcInstance(LocationDlgProc,hInst);
      lpfnGetClistDlg        = MakeProcInstance(GetCListDlgProc,hInst);
      lpfnAboutDlg           = MakeProcInstance(AboutDlgProc,hInst);
      break;
 
    case WM_COMMAND:
      switch (wParam)
      {
      case IDM_ALLOCATE:
        AllocateVerb = TRUE;
        if (DialogBox(hInst, "DB_ALLOCATE", hWnd, lpfnAllocateInfoDlg))
          {
            wsprintf(cBuffer,"Allocate Result = %d, %s",
                     globalret, (LPSTR)returnstring[globalret]);
            MessageBox(GetFocus(), cBuffer, "Command Result", MB_OK);
          }
        break;
 
      case IDM_EXTALLOCATE:
        AllocateVerb = FALSE;
        if (DialogBox(hInst, "DB_EXTALLOC", hWnd, lpfnExtAllocateInfoDlg))
          {
            if (DialogBox(hInst, "DB_ALLOCATE", hWnd, lpfnAllocateInfoDlg))
              {
                wsprintf(cBuffer,"EXT. Allocate Result = %d, %s",
                         globalret, (LPSTR)returnstring[globalret]);
                MessageBox(GetFocus(), cBuffer, "Command Result", MB_OK);
              }
          }
        break;
 
      case IDM_FLUSH:
        if (! SetCONID(hWnd))
          break;
        nReturnCode = DO_Flush(hWnd);
        wsprintf(cBuffer,"FLUSH Result = %d, %s",
                 nReturnCode, (LPSTR)returnstring[nReturnCode]);
        MessageBox(GetFocus(), cBuffer, "Command Result", MB_OK);
        break;
 
      case IDM_CONFIRM:
        if (! SetCONID(hWnd))
          break;
        nReturnCode = DO_Confirm(hWnd);
        wsprintf(cBuffer, "CONFIRM Result = %d, %s",
                 nReturnCode, (LPSTR)returnstring[nReturnCode]);
        MessageBox(GetFocus(), cBuffer, "Command Result", MB_OK);
        break;
 
      case IDM_CONFIRMED:
        if (! SetCONID(hWnd))
          break;
        nReturnCode = DO_Confirmed(hWnd);
        wsprintf(cBuffer,"CONFIRMED Result = %d, %s",
                 nReturnCode, (LPSTR)returnstring[nReturnCode]);
        MessageBox(GetFocus(), cBuffer, "Command Result", MB_OK);
        break;
 
      case IDM_SENDERROR:
        if (! SetCONID(hWnd))
          break;
        nReturnCode = DO_Send_Error(hWnd);
        wsprintf(cBuffer,"SEND ERROR Result = %d, %s",
                 nReturnCode, (LPSTR)returnstring[nReturnCode]);
        MessageBox(GetFocus(), cBuffer, "Command Result", MB_OK);
        break;
 
      case IDM_DEALLOCATE:
        if (! SetCONID(hWnd))
          break;
        nReturnCode = DialogBox(hInst, "DB_DEALLOCATE",hWnd,lpfnDeallocateDlg);
        if (nReturnCode)
          {
            nReturnCode = EHNAPPC_Deallocate(hWnd, conv_id, dealloctype);
            wsprintf(cBuffer, "DEALLOCATE Ret Code = %d, %s",
                     nReturnCode, (LPSTR)returnstring[nReturnCode]);
            MessageBox(GetFocus(), cBuffer, "Command Result", MB_OK);
 
            if (nReturnCode == EHNAPPC_OK)
              convlistarray[convindex].allocated = FALSE;
          }
        break;
 
 
      case IDM_PREPARETORECEIVE:
        if (! SetCONID(hWnd))
          break;
        nReturnCode = DO_Prepare_To_Receive(hWnd);
        wsprintf(cBuffer, "PREP To RCV Ret Code = %d, %s",
                 nReturnCode, (LPSTR)returnstring[nReturnCode]);
        MessageBox(GetFocus(), cBuffer, "Command Result", MB_OK);
        break;
 
      case IDM_REQUEST:
        if (! SetCONID(hWnd))
          break;
        nReturnCode = DO_Request_Send(hWnd);
        wsprintf(cBuffer, "REQ To SEND Ret Code = %d, %s",
                 nReturnCode, (LPSTR)returnstring[nReturnCode]);
        MessageBox(GetFocus(), cBuffer, "Command Result", MB_OK);
        break;
 
      case IDM_GETATTRIBUTES:
        if (! SetCONID(hWnd))
          break;
        nReturnCode = EHNAPPC_GetAttributes(hWnd, conv_id,
                                            (LPBYTE) &SyncLevel,
                                            (LPSTR) &modename,
                                            (LPSTR) &luname,
                                            (LPSTR) &hostname,
                                            (LPSTR) &userid);
 
        wsprintf(cBuffer,"GET ATTRIBUTES Ret Code = %d, %s",
                 nReturnCode, (LPSTR)returnstring[nReturnCode]);
        MessageBox(GetFocus(), cBuffer, "Command Result", MB_OK);
 
        if (nReturnCode == EHNAPPC_OK)
          {
            if (SyncLevel == EHNAPPC_SYNCLEVELNONE)
              wsprintf(cMsg, "Sync level = NONE.");
            else
              wsprintf(cMsg, "Sync level = CONFIRM.");
            wsprintf(cBuffer, "\nMode name = %s", (LPSTR)modename);
            lstrcat(cMsg, cBuffer);
            wsprintf(cBuffer, "\nRTLN entry = %s", (LPSTR)luname);
            lstrcat(cMsg, cBuffer);
            wsprintf(cBuffer, "\nHost name = %s", (LPSTR)hostname);
            lstrcat(cMsg, cBuffer);
            wsprintf(cBuffer, "\nUser id = %s", (LPSTR)userid);
            lstrcat(cMsg, cBuffer);
            MessageBox(GetFocus(), cMsg, "Get Attributes", MB_OK);
          }
        break;
 
      case IDM_ROUTERLOADED:
        if (EHNAPPC_IsRouterLoaded(hWnd))
          MessageBox(GetFocus(), "Router IS Loaded!!", "OK", MB_OK);
        else
          {
            lstrcpy(cMsg, "The Router IS NOT Loaded!!");
            lstrcat(cMsg, "\n\nAt least one of the following is TRUE:");
            lstrcat(cMsg, "\n\n1) The router really isn't loaded,");
            lstrcat(cMsg, "\n2) PCSWIN isn't loaded,");
            lstrcat(cMsg, "\n3) a down-level router is loaded.");
            MessageBox(GetFocus(), cMsg, "OOPS", MB_ICONSTOP);
          }
        break;
 
      case IDM_GETCAPABILITIES:
        /* build total structure */
        caplist.hdr.opcode             = EHNAPPC_OC_CAPABILITIES;
        caplist.hdr.length             = sizeof(caplist);
        caplist.mult.identifier        = EHNAPPC_CAP_OPTIMAL_COM_SIZE;
        caplist.mult.length            = sizeof(caplist.mult);
        caplist.qryconv.identifier     = EHNAPPC_CAP_QUERY_CONV_STATE;
        caplist.qryconv.length         = sizeof(caplist.qryconv);
        caplist.qryext.identifier      = EHNAPPC_CAP_EXT_ALLOCATE;
        caplist.qryext.length          = sizeof(caplist.qryext);
        caplist.qryfullsys.identifier  = EHNAPPC_CAP_QUERY_FULL_SYS;
        caplist.qryfullsys.length      = sizeof(caplist.qryext);
 
        nReturnCode = EHNAPPC_GetCapabilities(hWnd, (LPSTR) &caplist);
 
        wsprintf(cBuffer,"Get Capabilities return code = %d, %s",
                 nReturnCode, (LPSTR)returnstring[nReturnCode]);
        MessageBox(GetFocus(), cBuffer, "Command Result", MB_OK);
 
        if (nReturnCode == EHNAPPC_OK)
          {
            wsprintf(cMsg, "Optimal comm buffer multiplier = %d",
                     caplist.mult.data);
 
            if ((caplist.qryconv.rc   == EHNAPPC_CAP_SUCCESSFUL) &&
                (caplist.qryconv.data == EHNAPPC_CAP_SUPPORTED))
              wsprintf(cBuffer, "\nQuery conv state is supported.");
            else
              wsprintf(cBuffer, "\nQuery conv state is NOT supported.");
            lstrcat(cMsg, cBuffer);
 
            if ((caplist.qryext.rc   == EHNAPPC_CAP_SUCCESSFUL) &&
                (caplist.qryext.data == EHNAPPC_CAP_SUPPORTED))
              wsprintf(cBuffer, "\nExtended allocate is supported.");
            else
              wsprintf(cBuffer, "\nExtended allocate is NOT supported.");
            lstrcat(cMsg, cBuffer);
 
            if ((caplist.qryfullsys.rc   == EHNAPPC_CAP_SUCCESSFUL) &&
                (caplist.qryfullsys.data == EHNAPPC_CAP_SUPPORTED))
              wsprintf(cBuffer, "\nQueryFullSystems is supported.");
            else
              wsprintf(cBuffer, "\nQueryFullSystems is NOT supported.");
            lstrcat(cMsg, cBuffer);
 
            MessageBox(GetFocus(), cMsg, "Capability Info", MB_OK);
          }
        break;
 
      case IDM_QUERYSYSTEMS:
        SetCursor(LoadCursor(NULL,IDC_WAIT));
        DialogBox(hInst, "DB_LISTBOX", hWnd, lpfnSystemList);
        SetCursor(LoadCursor(NULL,IDC_ARROW));
        break;
 
      case IDM_QUERYFULLSYSTEMS:
        SetCursor(LoadCursor(NULL,IDC_WAIT));
        DialogBox(hInst, "DB_FULLSYSLISTBOX", hWnd, lpfnFullSystemList);
        SetCursor(LoadCursor(NULL,IDC_ARROW));
        break;
 
      case IDM_GETDEFAULTSYSTEM:
        nReturnCode = EHNAPPC_GetDefaultSystem(hWnd,
                                              (LPSTR) &luname);
        if (nReturnCode == EHNAPPC_OK)
          wsprintf(cBuffer, "Default system = %s.", (LPSTR)luname);
        else
          wsprintf(cBuffer, "Get Default System failed with RC = %d, %s",
                   nReturnCode, (LPSTR)returnstring[nReturnCode]);
        MessageBox(GetFocus(), cBuffer, "Get Default System", MB_OK);
        break;
 
      case IDM_QUERYCONVSTATE:
        if (! SetCONID(hWnd))
          break;
        nReturnCode = EHNAPPC_QueryConvState(hWnd, conv_id);
 
        if (nReturnCode == EHNAPPC_RESET_STATE)
          MessageBox(GetFocus(), "Conv State = RESET_STATE!",
                     "Command Result", MB_OK);
 
        else if (nReturnCode == EHNAPPC_SEND_STATE)
          MessageBox(GetFocus(), "Conv State = SEND_STATE!",
                     "Command Result", MB_OK);
 
        else if (nReturnCode == EHNAPPC_RECEIVE_STATE)
          MessageBox(GetFocus(), "Conv State = RECEIVE_STATE!",
                     "Command Result", MB_OK);
 
        else
          {
            wsprintf(cBuffer, "Conv State = %d.", nReturnCode);
            MessageBox(GetFocus(), cBuffer, "Command Result", MB_OK);
          }
        break;
 
      case IDM_QUERYUSERID:
        DialogBox(hInst, "DB_QUERYUID", hWnd, lpfnGetUserIDDlg);
        break;
 
      case IDM_A2E:
        if (! DialogBox(hInst, "DB_A2E", hWnd, lpfnA2eDlg))
          break;   /* User selected the cancel button */
 
        bytecount = lstrlen(AsciiBuffer);
        nReturnCode = EHNDT_ASCIIToEBCDIC(hWnd,
                                         (LPSTR) AsciiBuffer,
                                         (LPSTR) EbcdicBuffer,
                                         lstrlen(AsciiBuffer),
                                         (LPWORD) &bytecount);
        if (nReturnCode == EHNDT_SUCCESS)
          {
            wsprintf(cMsg, "The ASCII string is %d characters long.",
                           lstrlen(AsciiBuffer));
            wsprintf(cBuffer, "\n%d characters were translated to EBCDIC.",
                     bytecount);
            lstrcat(cMsg, cBuffer);
            MessageBox(GetFocus(), cMsg, "OK", MB_OK);
            EbcdicBuffer[bytecount] = '\0';
          }
        else
          MessageBox(GetFocus(), "A2E TRANSLATION FAILED!",
                     "A2E Translation Error", MB_OK);
        break;
 
      case IDM_E2A:
        bytecount = lstrlen(EbcdicBuffer);
        nReturnCode = EHNDT_EBCDICToASCII(hWnd,
                                         (LPSTR) EbcdicBuffer,
                                         (LPSTR) AsciiBuffer,
                                         lstrlen(EbcdicBuffer),
                                         (LPWORD) &bytecount);
        if (nReturnCode == EHNDT_SUCCESS)
          {
            wsprintf(cMsg, "The EBCDIC string is %d characters long.",
                           lstrlen(EbcdicBuffer));
            wsprintf(cBuffer, "\n%d characters were translated to ASCII.",
                     bytecount);
            lstrcat(cMsg, cBuffer);
            wsprintf(cBuffer, "\nThe ASCII string will follow in 100 byte chunks.");
            lstrcat(cMsg, cBuffer);
            MessageBox(GetFocus(), cMsg, "INFO", MB_OK);
 
            /*-------------------------------------------------------------*/
            /* Split the buffer into chunks, or else Windows will UAE for  */
            /* large amounts of data (ie. > 135 or so).                    */
            /*-------------------------------------------------------------*/
            for (i=0; i < bytecount; i+= 100)
             {
               memset(cMsg, '\0', sizeof(cMsg));
               memcpy(cMsg, AsciiBuffer+i, min(100, sizeof(AsciiBuffer)-i));
               MessageBox(GetFocus(), cMsg, "String", MB_OK);
             }
          }
        else
          MessageBox(GetFocus(), "E2A TRANSLATION FAILED!",
                     "E2A Translation Error", MB_OK);
        break;
 
 
      case IDM_TAKEMEMORY:
		//dxd  disabled memu item
        if (rmAllocated.dwBytes != 0L)
          {
            MessageBox(GetFocus(), "Memory is already allocated.",
                       "Sorry, Bud", MB_OK);
          }
        else
          {
            DialogBox(hInst, "DB_TAKEMEM", hWnd, lpfnTakeMemDlg);
          }
        break;
 
      case IDM_RESTOREMEMORY:
		//dxd  disabled memu item
        if (rmAllocated.dwBytes == 0L)
          {
            MessageBox(GetFocus(), "There's no memory to free.",
                                   "Free memory", MB_OK);
          }
        //else if (GlobalDosFree(rmAllocated.wSelector) == NULL)		
        //  {
        //    MessageBox(GetFocus(),"The memory was freed.","Free memory",MB_OK);
        //    rmAllocated.dwBytes = 0L;
        //  }
        else
          {
            MessageBox(GetFocus(), "The memory could NOT be freed!!!",
                       "ERROR", MB_OK);
          }
        break;
 
      case IDM_DOSAVAILABLE:
		 //dxd  disabled memu item
        if (GlobalDosLargest(MAX_MEM, MIN_MEM, &rmTemp))
          {
            wsprintf(cBuffer,"Largest DOS Block Available is %lu",
                     rmTemp.dwBytes);
            MessageBox(GetFocus(), cBuffer, "Largest block", MB_OK);
            //GlobalDosFree(rmTemp.wSelector);			
          }
        else
          {
            MessageBox(GetFocus(), "There is NO DOS memory available.",
                       "Largest block", MB_OK);
          }
        break;
 
 
      case IDM_PATTERN1:
      case IDM_PATTERN2:
      case IDM_PATTERN3:
      case IDM_PATTERN4:      /* Send data options */
      case IDM_PATTERN5:
      case IDM_PATTERN6:
      case IDM_PATTERN7:
      case IDM_PATTERN8:
        if (! SetCONID(hWnd))
          break;
 
        patternnum = wParam - IDM_PATTERN1;
        init_pattern1();
 
        /* 1st send the "Send" string */
        lstrcpy(pattern1->data, "Send");
        pattern1->llfield[0] = 00;
        pattern1->llfield[1] = lstrlen((LPSTR)pattern1->data) + LL_HDR;
        bytecount = sizeof(pattern1->data);
        EHNDT_ASCIIToEBCDIC(hWnd,
                           (LPSTR) pattern1->data,
                           (LPSTR) pattern1->data,
                           lstrlen((LPSTR)pattern1->data),
                           &bytecount);
        nReturnCode = EHNAPPC_SendData(hWnd, conv_id,
                                       pattern1->llfield[1],
                                       (LPVOID) pattern1,
                                       (LPBYTE) &globalRTS);
        if (nReturnCode != EHNAPPC_OK)
          {
            wsprintf(cBuffer,"Send Data failed with ret code %d, %s",
                     nReturnCode, (LPSTR)returnstring[nReturnCode]);
            MessageBox(GetFocus(), cBuffer, "Command Result", MB_OK);
            break;
          }
 
        /* next send the LL Pattern Indicator  */
        pattern1->llfield[0] = 00;
        pattern1->llfield[1] = LL_HDR + 1;     /* LL hdr + 1 byte of data */
        pattern1->data[0] = (char) (patternnum+1); /* send pattern number */
        nReturnCode = EHNAPPC_SendData(hWnd, conv_id,
                                       pattern1->llfield[1],
                                       (LPVOID) pattern1,
                                       (LPBYTE) &globalRTS);
 
        /* send the actual data */
        if (patternnum < 6)
          {
            init_pattern1();
            pattern1->llfield[0] = (pattern_length[patternnum]+LL_HDR) / 0x100;
            pattern1->llfield[1] = (pattern_length[patternnum]+LL_HDR) % 0x100;
            memcpy(pattern1->data, pattern_buffer[patternnum],
                     pattern_length[patternnum]);
 
            nReturnCode = EHNAPPC_SendData(hWnd, conv_id,
                                           pattern_length[patternnum] + LL_HDR,
                                           (LPVOID) pattern1,
                                           (LPBYTE) &globalRTS);
          }
        else /* either pattern 7 or 8 */
          {
            DataPtr->llfield[0]  = (pattern_length[patternnum]+LL_HDR) / 0x100;
            DataPtr->llfield[1]  = (pattern_length[patternnum]+LL_HDR) % 0x100;
            if (patternnum == 6) /* actually 7 chosen    */
              {
                /* fill with 55AA */
                for (i=0; i < pattern_7_length; i++)
                  DataPtr->data[i] = (i%2 ? 0x55 : 0xaa);
              }
            else                 /* pattern 8 was chosen */
              {
                /* fill with AA55 */
                for (i=0; i < pattern_8_length; i++)
                  DataPtr->data[i] = (i%2 ? 0xaa : 0x55);
              }
 
            nReturnCode = EHNAPPC_SendData(hWnd, conv_id,
                                           pattern_length[patternnum] + LL_HDR,
                                           (LPVOID) DataPtr,
                                           (LPBYTE) &globalRTS);
 
          } /* end else leg of if patternnum <6 */
 
        wsprintf(cBuffer,"Send Return Code = %d, %s",
                 nReturnCode, (LPSTR)returnstring[nReturnCode]);
        MessageBox(GetFocus(), cBuffer, "Command Result", MB_OK);
        if (nReturnCode != EHNAPPC_OK)  /* Leave conversation in send state */
          break;
 
        nReturnCode = EHNAPPC_PrepareToReceive(hWnd, conv_id);
 
        init_pattern1();
 
        /* Receive indication from host whether it received the data OK. */
        nReturnCode = EHNAPPC_ReceiveAndWait(hWnd, conv_id,
                                             EHNAPPC_LL,
                                             32,
                                             (LPVOID) pattern1,
                                             (LPBYTE) &what,
                                             (LPBYTE) &reqsToSend,
                                             (LPWORD) &receivedLength);
        if (nReturnCode != EHNAPPC_OK)
          {
            wsprintf(cBuffer,"Rcv and Wait RC = %d, %s",
                     nReturnCode, (LPSTR)returnstring[nReturnCode]);
            MessageBox(GetFocus(), cBuffer, "ERROR", MB_OK);
            break;
          }
        if (debug)
          {
            wsprintf(cBuffer, "What Received = %d, Rcv and Wait RC = %d, %s",
                     what, nReturnCode, (LPSTR)returnstring[nReturnCode]);
            MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
          }
 
        /* Host sends the position of the string that had an error. 0 = OK */
        if ((pattern1->data[0] == 0) && (pattern1->data[1] == 0))
          {
            wsprintf(cBuffer,"HOST Received Data OK ");
            MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
          }
        else  /* host saw an error in the data */
          {
            wsprintf(cBuffer, "HOST saw an ERROR at offset %d",
                     256*(pattern1->data[0]) + (pattern1->data[1]));
            MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
          }
 
        /* Must issue second receive to get Send indicator. */
        nReturnCode = EHNAPPC_ReceiveAndWait(hWnd, conv_id,
                                             EHNAPPC_LL,
                                             0,
                                             (LPVOID) &command,
                                             (LPBYTE) &what,
                                             (LPBYTE) &reqsToSend,
                                             (LPWORD) &receivedLength);
        if (nReturnCode != EHNAPPC_OK)
          {
            wsprintf(cBuffer,"Rcv and Wait RC = %d, %s",
                     nReturnCode, (LPSTR)returnstring[nReturnCode]);
            MessageBox(GetFocus(), cBuffer, "ERROR", MB_OK);
          }
 
        break;
 
 
      case IDM_PATTERN1R:
      case IDM_PATTERN2R:
      case IDM_PATTERN3R:
      case IDM_PATTERN4R:    /* Receive Immediate menu options */
      case IDM_PATTERN5R:
      case IDM_PATTERN6R:
      case IDM_PATTERN7R:
      case IDM_PATTERN8R:
        if (! SetCONID(hWnd))
          break;
 
        patternnum = wParam - IDM_PATTERN1R;
        MaxLength = pattern_length[patternnum] + LL_HDR;
        init_pattern1();
 
        /*-------------------------------------------------------------*/
        /* Determine whether the receive buffer should be in extended  */
        /* or conventional memory. This is NOT the comm buffer, it's   */
        /* the RECEIVE buffer.  If it's in conventional memory, you    */
        /* get better performance because you avoid a copy in the DLL. */
        /*-------------------------------------------------------------*/
        if (! DialogBox(hInst, "DB_LOCATION", hWnd, lpfnLocationDlg))
          break;                        /* User selected CANCEL button */
 
        /* 1st send the "Receive" string */
        lstrcpy(pattern1->data, "Receive");
        pattern1->llfield[0] = 00;
        pattern1->llfield[1] = lstrlen((LPSTR)pattern1->data) + LL_HDR;
        bytecount = sizeof(pattern1->data);
        EHNDT_ASCIIToEBCDIC(hInst,
                           (LPSTR) pattern1->data,
                           (LPSTR) pattern1->data,
                           lstrlen((LPSTR)pattern1->data),
                           &bytecount);
        nReturnCode = EHNAPPC_SendData(hWnd, conv_id,
                                       pattern1->llfield[1],
                                       (LPVOID) pattern1,
                                       (LPBYTE) &globalRTS);
 
        /* next send the LL Pattern Indicator  */
        pattern1->llfield[0] = 00;
        pattern1->llfield[1] = LL_HDR + 1;     /* LL hdr + 1 byte of data */
        pattern1->data[0] = (char) (patternnum+1); /* send pattern number */
        nReturnCode = EHNAPPC_SendData(hWnd, conv_id,
                                       pattern1->llfield[1],
                                       (LPVOID) pattern1,
                                       (LPBYTE) &globalRTS);
 
        /* Return if PrepareToReceive fails. Do it here instead of */
        /* after the sends so it's not as confusing to the user.   */
        nReturnCode = EHNAPPC_PrepareToReceive(hWnd, conv_id);
        if (nReturnCode != EHNAPPC_OK)
          {
            wsprintf(cBuffer, "Prepare to receive failed with rc = %d, %s",
                     nReturnCode, (LPSTR)returnstring[nReturnCode]);
            MessageBox(GetFocus(), cBuffer, "ERROR", MB_ICONSTOP);
            break;
          }
 
        if (Location == EXTENDED)       /* Use the extended memory that */
          {                             /*   was already allocated.     */
            tempptr = (LPSTR) DataPtr;
            rmTemp.dwBytes = 0;
          }
        else                            /* Allocate conventional memory */
          {
            if (! GlobalDosLargest(MaxLength, MaxLength, &rmTemp))
              {
                MessageBox(GetFocus(),
                           "Failed to allocate memory for receive buffer.",
                           "MEMORY ALLOCATION ERROR", MB_ICONSTOP);
                break;
              }
            tempptr = (LPVOID) MAKELONG(0, rmTemp.wSelector);
          }
 
        /*--------------------------------------------------------------*/
        /* Receive the data.                                            */
        /*--------------------------------------------------------------*/
        TotalLength = 0;
        do {
          nReturnCode = EHNAPPC_ReceiveImmediate(hWnd, conv_id,
                                                 EHNAPPC_LL,
                                                 MaxLength,
                                                 (LPVOID) tempptr,
                                                 (LPBYTE) &what,
                                                 (LPBYTE) &reqsToSend,
                                                 (LPWORD) &receivedLength);
          if (nReturnCode == EHNAPPC_OK)
            {
              TotalLength += receivedLength;  /* bump count */
              tempptr += receivedLength;      /* bump where to put it */
              wsprintf(cBuffer,"Router Gave Data, Running Total = %d",
                       TotalLength);
              MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
            }
          else if (nReturnCode != EHNAPPC_UNSUCCESSFUL)
            break;         /* BAD result--don't receive anymore. */
        } while ((nReturnCode == EHNAPPC_UNSUCCESSFUL) ||
                ((what != EHNAPPC_DATACOMPLETE) &&
                 (what != EHNAPPC_RECEIVEDSEND)));
 
        if (nReturnCode == EHNAPPC_OK)
          {
            wsprintf(cBuffer, "What Rcvd = %d, Rcv Immediate RC = %d, %s",
                     what, nReturnCode, (LPSTR)returnstring[nReturnCode]);
            MessageBox(GetFocus(), cBuffer, "Command Result", MB_OK);
          }
        else
          {
            wsprintf(cBuffer, "Rcv Immediate RC = %d, %s",
                     nReturnCode, (LPSTR)returnstring[nReturnCode]);
            MessageBox(GetFocus(), cBuffer, "Command Result", MB_ICONSTOP);
            break;
          }
 
        if (debug)
          {
            wsprintf(cBuffer, "Total number of bytes = %d", TotalLength);
            MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
          }
 
        /* The verify routine expects the data in the DataPtr structure */
        if (Location == REAL)
          memcpy(DataPtr, (LPSTR) MAKELONG(0, rmTemp.wSelector),
                   TotalLength);
 
        nReturnCode = verify_pattern(patternnum, TotalLength);
        if (nReturnCode == GOOD_DATA)
          MessageBox(GetFocus(), "PC Received Data OK", "INFO", MB_OK);
        else if (nReturnCode == BAD_LENGTH)
          MessageBox(GetFocus(),"Error in Length of Data!","DATA ERROR!",MB_OK);
        else
          {
            wsprintf (cBuffer, "Error in Data at Position %d", nReturnCode);
            MessageBox(GetFocus(), cBuffer, "DATA ERROR!", MB_OK);
          }
 
        /* Must issue second receive to get Send indicator. */
        nReturnCode = EHNAPPC_ReceiveAndWait(hWnd, conv_id,
                                             EHNAPPC_LL,
                                             0,
                                             (LPVOID) &command,
                                             (LPBYTE) &what,
                                             (LPBYTE) &reqsToSend,
                                             (LPWORD) &receivedLength);
        if (nReturnCode != EHNAPPC_OK)
          {
            wsprintf(cBuffer,"RCV and Wait RC = %d, %s",
                     nReturnCode, (LPSTR)returnstring[nReturnCode]);
            MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
          }
 
        if (rmTemp.dwBytes > 0)
          //GlobalDosFree(rmTemp.wSelector);
			
 
        break;
 
 
      case IDM_PATTERN1RW:
      case IDM_PATTERN2RW:
      case IDM_PATTERN3RW:
      case IDM_PATTERN4RW:   /* Receive and Wait menu options */
      case IDM_PATTERN5RW:
      case IDM_PATTERN6RW:
      case IDM_PATTERN7RW:
      case IDM_PATTERN8RW:
        if (! SetCONID(hWnd))
          break;
 
        patternnum = wParam - IDM_PATTERN1RW;
        MaxLength = pattern_length[patternnum] + LL_HDR;
        init_pattern1();
 
        /* Determine whether the receive buffer should be in extended   */
        /* or conventional memory.                                      */
        if (! DialogBox(hInst, "DB_LOCATION", hWnd, lpfnLocationDlg))
          break;                         /* User selected CANCEL button */
 
        /* 1st send the "Receive" string */
        lstrcpy(pattern1->data, "Receive");
        pattern1->llfield[0] = 00;
        pattern1->llfield[1] = lstrlen((LPSTR)pattern1->data) + LL_HDR;
        bytecount = sizeof(pattern1->data);
        EHNDT_ASCIIToEBCDIC(hWnd,
                            (LPSTR) pattern1->data,
                            (LPSTR) pattern1->data,
                            lstrlen((LPSTR)pattern1->data),
                            &bytecount);
        nReturnCode = EHNAPPC_SendData(hWnd, conv_id,
                                       pattern1->llfield[1],
                                       (LPVOID) pattern1,
                                       (LPBYTE) &globalRTS);
 
        /* next send the LL Pattern Indicator  */
        pattern1->llfield[0] = 00;
        pattern1->llfield[1] = LL_HDR + 1;     /* LL hdr + 1 byte of data */
        pattern1->data[0] = (char) (patternnum+1); /* send pattern number */
        nReturnCode = EHNAPPC_SendData(hWnd,conv_id,
                                       pattern1->llfield[1],
                                       (LPVOID) pattern1,
                                       (LPBYTE) &globalRTS);
 
        /* Return if prepare to receive fails. Do it here instead of after */
        /* the send requests so it's not as confusing to the user.         */
        nReturnCode = EHNAPPC_PrepareToReceive(hWnd, conv_id);
        if (nReturnCode != EHNAPPC_OK)
          {
            wsprintf(cBuffer,"Prepare to receive failed with ret code %d, %s",
                     nReturnCode, (LPSTR)returnstring[nReturnCode]);
            MessageBox(GetFocus(), cBuffer, "ERROR", MB_ICONSTOP);
            break;
          }
 
        if (Location == EXTENDED)
          {
            tempptr = (LPSTR) DataPtr;
            rmTemp.dwBytes = 0;
          }
        else
          {
            if (! GlobalDosLargest(MaxLength, MaxLength, &rmTemp))
              {
                MessageBox(GetFocus(),
                           "Failed to allocate memory for receive buffer.",
                           "MEMORY ALLOCATION ERROR", MB_ICONSTOP);
                break;
              }
            tempptr = (LPVOID) MAKELONG(0, rmTemp.wSelector);
          }
 
        nReturnCode = EHNAPPC_ReceiveAndWait(hWnd, conv_id,
                                             EHNAPPC_LL,
                                             MaxLength,
                                             (LPVOID) tempptr,
                                             (LPBYTE) &what,
                                             (LPBYTE) &reqsToSend,
                                             (LPWORD) &receivedLength);
 
        wsprintf(cBuffer,"What Received = %d, Rcv and Wait RC = %d, %s",
                 what, nReturnCode, (LPSTR)returnstring[nReturnCode]);
        if (nReturnCode == EHNAPPC_OK)
          MessageBox(GetFocus(), cBuffer, "Command Result", MB_OK);
        else
          {
            MessageBox(GetFocus(), cBuffer, "Command Result", MB_ICONSTOP);
            break;
          }
 
        if (debug)
          {
            wsprintf(cBuffer, "# Bytes = %d", receivedLength);
            MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
          }
 
        /* The verify routine expects the data in the DataPtr struct */
        if (Location == REAL)
          {
            memcpy(DataPtr, tempptr, receivedLength);
          }
 
        nReturnCode = verify_pattern(patternnum, receivedLength);
        if (nReturnCode == GOOD_DATA)
          MessageBox(GetFocus(), "PC Received Data OK", "INFO",MB_OK);
        else if (nReturnCode == BAD_LENGTH)
          MessageBox(GetFocus(),"Error in Length of Data!","DATA ERROR!",MB_OK);
        else
          {
            wsprintf(cBuffer, "Error in Data at Offset %d", nReturnCode);
            MessageBox(GetFocus(), cBuffer, "DATA ERROR!", MB_OK);
          }
 
        /* Must issue second receive to get Send indicator. */
        nReturnCode = EHNAPPC_ReceiveAndWait(hWnd, conv_id,
                                             EHNAPPC_LL,
                                             0,
                                             (LPVOID) &command,
                                             (LPBYTE) &what,
                                             (LPBYTE) &reqsToSend,
                                             (LPWORD) &receivedLength);
        if (nReturnCode != EHNAPPC_OK)
          {
            wsprintf(cBuffer, "RCV and Wait RC = %d, %s",
                     nReturnCode, (LPSTR)returnstring[nReturnCode]);
            MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
          }
 
        if (rmTemp.dwBytes > 0)
		  //GlobalDosFree(rmTemp.wSelector);
		   
 
        break;
 
 
      /*---------------------------------------------------------------*/
      /* This option allows the user to build a custom data pattern to */
      /* send to the host. The host then sends it back and we verify   */
      /* that it gets sent back and forth ok.                          */
      /*---------------------------------------------------------------*/
      case IDM_CUSTOMPATTERN:
        if (! SetCONID(hWnd))
          break;
 
        if (! DialogBox(hInst, "DB_CUSTOMDATA", hWnd, lpfnCustomDataDlg))
          break;   /* User selected the cancel button */
 
        DataPtr->llfield[0] = (CustomLLs.TotalBytes) / 0x100;
        DataPtr->llfield[1] = (CustomLLs.TotalBytes) % 0x100;
        DataByte = 0; /* index into Byte array */
        for (i=1; i <= CustomLLs.NumOfLLs; ++i) /* build the LL subfields */
         {
           /* take care of LL-Header 1st */
           DataPtr->data[DataByte]   = (CustomLLs.LLinfo[i-1].Size) / 0x100;
           DataPtr->data[DataByte+1] = (CustomLLs.LLinfo[i-1].Size) % 0x100;
           DataByte = DataByte + LL_HDR;  /* now point to data subfield */
 
           /* next take care of Data Portion */
           /* subtract the sub-LL Header     */
           for (j=0; j < (CustomLLs.LLinfo[i-1].Size-LL_HDR); j++)
             {
                /* fill in subfield Data */
                if (DataByte > 32767)
                  break; /* break out if too big */
                DataPtr->data[DataByte] = CustomLLs.LLinfo[i-1].Start;
 
                DataByte ++ ;     /* point to next byte in the array */
                CustomLLs.LLinfo[i-1].Start ++ ; /* inc to next byte */
             } /* end for j statement */
         } /* end for i statement */
 
        if (CustomLLs.MemLocation == EXTENDED) /* use CustomPtr to send */
          {
            CustomPtr = DataPtr;
            rmTemp.dwBytes = 0L;
          }
        else                       /* use conventional memory for speed */
          {
            if (! GlobalDosLargest(CustomLLs.TotalBytes+LL_HDR,
                                   CustomLLs.TotalBytes+LL_HDR, &rmTemp))
              {
                MessageBox(GetFocus(),
                       "Failed to allocate memory for custom data!",
                       "TEST DRIVER MEMORY ALLOCATION ERROR", MB_ICONSTOP);
                break;
              }
             CustomPtr = (struct BigPattern  far *)
                         MAKELONG(0, rmTemp.wSelector);
             memcpy(CustomPtr, DataPtr, LOWORD(CustomLLs.TotalBytes));
          }
 
 
        /* First, tell the host what's coming; send the "Send" string. */
        init_pattern1();
        lstrcpy(pattern1->data, "Send");
        pattern1->llfield[0] = 00;
        pattern1->llfield[1] = lstrlen((LPSTR)pattern1->data) + LL_HDR;
        bytecount = sizeof(pattern1->data);
        EHNDT_ASCIIToEBCDIC(hWnd,
                            (LPSTR) pattern1->data,
                            (LPSTR) pattern1->data,
                            lstrlen((LPSTR)pattern1->data),
                            &bytecount);
        nReturnCode = EHNAPPC_SendData(hWnd, conv_id,
                                       pattern1->llfield[1],
                                       (LPVOID) pattern1,
                                       (LPBYTE) &globalRTS);
        if (nReturnCode != EHNAPPC_OK)
          {
            wsprintf(cBuffer,"Send Data failed with ret code = %d, %s",
                     nReturnCode, (LPSTR)returnstring[nReturnCode]);
            MessageBox(GetFocus(), cBuffer, "ERROR", MB_ICONSTOP);
            break;
          }
 
        /* next send the LL Pattern Indicator  */
        pattern1->llfield[0] = 00;
        pattern1->llfield[1] = LL_HDR + 1;     /* LL hdr + 1 byte of data */
        pattern1->data[0] = (char) (00); /* send Custom Pattern indicator */
        nReturnCode = EHNAPPC_SendData(hWnd, conv_id,
                                       pattern1->llfield[1],
                                       (LPVOID) pattern1,
                                       (LPBYTE) &globalRTS);
 
        /* send the custom Data Pattern */
        nReturnCode = EHNAPPC_SendData(hWnd, conv_id,
                                       LOWORD(CustomLLs.TotalBytes),
                                       (LPVOID) CustomPtr,
                                       (LPBYTE) &globalRTS);
        if (nReturnCode != EHNAPPC_OK)
          {
            wsprintf(cBuffer,"Send Data failed with ret code = %d, %s",
                     nReturnCode, (LPSTR)returnstring[nReturnCode]);
            MessageBox(GetFocus(), cBuffer, "Command Result", MB_ICONSTOP);
            if (rmTemp.dwBytes > 0)
              //GlobalDosFree(rmTemp.wSelector);
			  
            break;
          }
 
        wsprintf(cBuffer,"Send Return Code = %d, %s",
                 nReturnCode, (LPSTR)returnstring[nReturnCode]);
        MessageBox(GetFocus(), cBuffer, "Command Result", MB_OK);
 
        nReturnCode = EHNAPPC_PrepareToReceive(hWnd,conv_id);
 
        if (debug)
          {
            MessageBox(GetFocus(), "Now Verify Pattern!", "OK", MB_OK);
          }
 
        /* Use RcvPtr to receive the data back from the host */
        hTempRcv = GlobalAlloc(GMEM_MOVEABLE | GMEM_NODISCARD |
                               GMEM_ZEROINIT, CustomLLs.TotalBytes);
        RcvPtr = (struct BigPattern  far *) GlobalLock(hTempRcv);
        if ((hTempRcv == 0) || (RcvPtr == NULL))
          {
            lstrcpy(cMsg, "Failed to allocate/lock buffer to receive data ");
            lstrcat(cMsg, "back from host!");
            lstrcat(cMsg, "\nDeallocate this conversation with Type=ABEND.");
            lstrcat(cMsg, "\nFree some memory and try again.");
            MessageBox(GetFocus(), cMsg,
                       "TEST DRIVER MEMORY ALLOCATION ERROR", MB_ICONSTOP);
            if (rmTemp.dwBytes > 0)
              //GlobalDosFree(rmTemp.wSelector);
			  
            break;
          }
 
        /* wait for Host to echo the Data Back */
        nReturnCode = EHNAPPC_ReceiveAndWait(hWnd, conv_id,
                                             EHNAPPC_LL,
                                             LOWORD(CustomLLs.TotalBytes),
                                             (LPVOID) RcvPtr,
                                             (LPBYTE) &what,
                                             (LPBYTE) &reqsToSend,
                                             (LPWORD) &receivedLength);
        if (nReturnCode != EHNAPPC_OK)
          {
            wsprintf(cBuffer,"RCV-Wait failed with RC = %d, %s",
                     nReturnCode, (LPSTR)returnstring[nReturnCode]);
            MessageBox(GetFocus(), cBuffer, "Command Result", MB_ICONSTOP);
            if (rmTemp.dwBytes > 0)
              //GlobalDosFree(rmTemp.wSelector);
			  
            break;
          }
        else if (debug)
          {
            wsprintf(cBuffer,"What Received = %d, RCV-Wait RC = %d, %s",
                     what, nReturnCode, (LPSTR)returnstring[nReturnCode]);
            MessageBox(GetFocus(), cBuffer, "Host Echo Data", MB_OK);
 
            wsprintf(cBuffer, "# Bytes = %d", receivedLength);
            MessageBox(GetFocus(), cBuffer, "Host Echo Data", MB_OK);
          }
 
        if (receivedLength == CustomLLs.TotalBytes)
          {
            m = (LPSTR) CustomPtr;
            n = (LPSTR) RcvPtr;
            for (i=0; i < receivedLength; i++)
              {
                if (m[i] != n[i])
                  break;
              }
            if (i== receivedLength)
              MessageBox(GetFocus(), "Custom Data OK!", "OK", MB_OK);
            else
              {
                wsprintf(cMsg, "Custom Data error at position %d!", i+1);
                lstrcat(cMsg, "\nHost did not resend the data correctly!");
                MessageBox(GetFocus(), cMsg, "ERROR", MB_OK);
              }
          }
        else
          {
            wsprintf(cBuffer,"Custom Data error! Only received %d bytes back.",
                     receivedLength);
            MessageBox(GetFocus(), cBuffer, "ERROR", MB_OK);
          }
 
        /* Must issue second receive to get Send indicator. */
        nReturnCode = EHNAPPC_ReceiveAndWait(hWnd, conv_id,
                                             EHNAPPC_LL,
                                             0,
                                             (LPVOID) &command,
                                             (LPBYTE) &what,
                                             (LPBYTE) &reqsToSend,
                                             (LPWORD) &receivedLength);
 
        GlobalUnlock(hTempRcv);
        GlobalFree(hTempRcv);  /* free up memory */
 
        if (rmTemp.dwBytes > 0)
          //GlobalDosFree(rmTemp.wSelector);
		   
 
        break;
 
 
      /*---------------------------------------------------------------*/
      /* This option allows the user to build a custom data pattern to */
      /* receive from the host. We tell the host how many LL's to send */
      /* to us and the length we want, and the host does it.           */
      /*---------------------------------------------------------------*/
      case IDM_CUSTOMPATTERNRI:
      case IDM_CUSTOMPATTERNRW:
        if (! SetCONID(hWnd))
          break;
 
        if (! DialogBox(hInst, "DB_CUSTOMRCV", hWnd, lpfnCustomRcvDlg))
          break;   /* User selected the cancel button */
 
        if (CustomRcv.MemLocation == EXTENDED) /* use CustomPtr to receive */
          {
            CustomPtr = DataPtr;
            rmTemp.dwBytes = 0L;
          }
        else                       /* use real memory */
          {
            if (! GlobalDosLargest(CustomRcv.Size,
                                   CustomRcv.Size, &rmTemp))
              {
                MessageBox(GetFocus(), "Failed to allocate memory for custom data!",
                           "TEST DRIVER MEMORY ALLOCATION ERROR", MB_ICONSTOP);
                break;
              }
            CustomPtr = (struct BigPattern far *) MAKELONG(0, rmTemp.wSelector);
          }
 
        if (debug)
          MessageBox(GetFocus(), "Tell the host what to do.", "OK", MB_OK);
 
        /* 1st send the "Receive" string */
        init_pattern1();
        lstrcpy(pattern1->data, "Receive");
        pattern1->llfield[0] = 0;
        pattern1->llfield[1] = lstrlen(pattern1->data) + LL_HDR;
        bytecount = sizeof(pattern1->data);
        EHNDT_ASCIIToEBCDIC(hWnd,
                            (LPSTR) pattern1->data,
                            (LPSTR) pattern1->data,
                            lstrlen(pattern1->data),
                            &bytecount);
        nReturnCode = EHNAPPC_SendData(hWnd, conv_id,
                                       pattern1->llfield[1],
                                       (LPVOID) pattern1,
                                       (LPBYTE) &globalRTS);
        if (nReturnCode != EHNAPPC_OK)
          {
            wsprintf(cBuffer,"Send Data failed with ret code = %d, %s",
                     nReturnCode, (LPSTR)returnstring[nReturnCode]);
            MessageBox(GetFocus(), cBuffer, "ERROR", MB_ICONSTOP);
            break;
          }
 
        /* next send the LL Pattern Indicator  */
        pattern1->llfield[0] = 0;
        pattern1->llfield[1] = LL_HDR + 1;    /* LL hdr + 1 byte of data */
        pattern1->data[0] = 0;          /* send Custom Pattern indicator */
        nReturnCode = EHNAPPC_SendData(hWnd, conv_id,
                                       pattern1->llfield[1],
                                       (LPVOID) pattern1,
                                       (LPBYTE) &globalRTS);
 
        /* Next tell the host how many LLs to send and the length of */
        /* each LL.  The values must be byte-reversed for the host.  */
        pattern1->llfield[0] = 0;
        pattern1->llfield[1] = 6;
        pattern1->data[0] = CustomRcv.NumOfLLs / 0x100;
        pattern1->data[1] = CustomRcv.NumOfLLs % 0x100;
        pattern1->data[2] = CustomRcv.Size / 0x100;
        pattern1->data[3] = CustomRcv.Size % 0x100;
        nReturnCode = EHNAPPC_SendData(hWnd, conv_id,
                                       pattern1->llfield[1],
                                       (LPVOID) pattern1,
                                       (LPBYTE) &globalRTS);
        if (nReturnCode != EHNAPPC_OK)
          {
            wsprintf(cBuffer,"Send Data failed with ret code = %d, %s",
                     nReturnCode, (LPSTR)returnstring[nReturnCode]);
            MessageBox(GetFocus(), cBuffer, "Command Result", MB_ICONSTOP);
            if (rmTemp.dwBytes > 0)
              //GlobalDosFree(rmTemp.wSelector);
			   
            break;
          }
 
        if (debug)
          {
            MessageBox(GetFocus(), "Now receive the data from the host!",
                       "OK", MB_OK);
          }
 
        nReturnCode = EHNAPPC_PrepareToReceive(hWnd,conv_id);
        if (nReturnCode != EHNAPPC_OK)
          {
            wsprintf(cBuffer,"Prepare to receive failed with RC = %d, %s",
                     nReturnCode, (LPSTR)returnstring[nReturnCode]);
            MessageBox(GetFocus(), cBuffer, "Command Result", MB_ICONSTOP);
            if (rmTemp.dwBytes > 0)
              //GlobalDosFree(rmTemp.wSelector);
			  
            break;
          }
 
        /* Build a string to compare against the data received */
        /* from the host to make sure it was sent correctly.   */
        hTempRcv = GlobalAlloc(GMEM_MOVEABLE | GMEM_NODISCARD |
                               GMEM_ZEROINIT, CustomRcv.Size);
        RcvPtr = (struct BigPattern  far *) GlobalLock(hTempRcv);
        if ((hTempRcv == 0) || (RcvPtr == NULL))
          {
            lstrcpy(cMsg, "Failed to allocate/lock buffer to receive data ");
            lstrcat(cMsg, "from host!");
            lstrcat(cMsg, "\nDeallocate this conversation with Type=ABEND.");
            lstrcat(cMsg, "\nFree some memory and try again.");
            MessageBox(GetFocus(), cMsg,
                       "TEST DRIVER MEMORY ALLOCATION ERROR", MB_ICONSTOP);
            if (rmTemp.dwBytes > 0)
              //GlobalDosFree(rmTemp.wSelector);
			  
            break;
          }
        for (i=0; i < CustomRcv.Size-LL_HDR; i++)  /* Match the string the */
          RcvPtr->data[i] = i % 0x100;             /*   host creates.      */
 
        /*-----------------------------------------------------------------*/
        /* Receive each LL from the host and ensure it was sent correctly  */
        /*-----------------------------------------------------------------*/
        for (i=0; i < CustomRcv.NumOfLLs; i++)
         {
          CustomPtr->llfield[0] = 0;       /* Clear out the receive buffer */
          CustomPtr->llfield[1] = 0;
          for (j=0; j < CustomRcv.Size-LL_HDR; j++)
            CustomPtr->data[j] = '\0';
 
          if (debug)
            {
              wsprintf(cBuffer, "Attempt to receive LL #%d.", i+1);
              MessageBox(GetFocus(), cBuffer, "Info", MB_ICONINFORMATION);
            }
 
          if (wParam == IDM_CUSTOMPATTERNRI)      /* Use receive immediate */
            {
              tempptr = (LPSTR) CustomPtr;
              receivedLength = 0;  /* Number of bytes received for this LL */
              do {
                nReturnCode = EHNAPPC_ReceiveImmediate(hWnd,conv_id,
                                              EHNAPPC_LL,
                                              LOWORD(CustomRcv.Size),
                                              (LPVOID) tempptr,
                                              (LPBYTE) &what,
                                              (LPBYTE) &reqsToSend,
                                              (LPWORD) &bytecount);
                if (debug)
                  {
                    wsprintf(cBuffer, "Receive immediate RC = %d, %s.",
                             nReturnCode, (LPSTR)returnstring[nReturnCode]);
                    MessageBox(GetFocus(), cBuffer, "Host Sent Data", MB_OK);
                  }
                if (nReturnCode == EHNAPPC_OK)
                  {
                    receivedLength += bytecount;   /* bump this LL count   */
                    tempptr += bytecount;          /* bump where to put it */
                    if (debug)        /* These fields are undefined unless */
                      {               /*   the rc equals EHNAPPC_OK        */
                        wsprintf(cMsg, "# bytes received this time = %d.",
                                 bytecount);
                        wsprintf(cBuffer, "\nWhat Received = %d.", what);
                        lstrcat(cMsg, cBuffer);
                        MessageBox(GetFocus(), cMsg, "Host Sent Data", MB_OK);
                      }
                  }
                else if (nReturnCode != EHNAPPC_UNSUCCESSFUL)
                  {
                    wsprintf(cMsg, "Rcv Immed failed with RC = %d, %s.",
                             nReturnCode, (LPSTR)returnstring[nReturnCode]);
                    lstrcat(cMsg, "\nMenu option aborted!");
                    MessageBox(GetFocus(), cMsg, "ERROR", MB_ICONSTOP);
                    break;           /* BAD result--don't receive anymore. */
                  }
              } while ((nReturnCode == EHNAPPC_UNSUCCESSFUL) ||
                      ((what != EHNAPPC_DATACOMPLETE) &&
                       (what != EHNAPPC_RECEIVEDSEND)));
            } /* end if receive immediate */
 
          else       /* Use receive and wait */
            {
              nReturnCode = EHNAPPC_ReceiveAndWait(hWnd, conv_id,
                                                   EHNAPPC_LL,
                                                   LOWORD(CustomRcv.Size),
                                                   (LPVOID) CustomPtr,
                                                   (LPBYTE) &what,
                                                   (LPBYTE) &reqsToSend,
                                                   (LPWORD) &receivedLength);
              if (nReturnCode != EHNAPPC_OK)
                {
                  wsprintf(cMsg, "Rcv-Wait failed with RC = %d, %s.",
                           nReturnCode, (LPSTR)returnstring[nReturnCode]);
                  lstrcat(cMsg, "\nMenu option aborted!");
                  MessageBox(GetFocus(), cMsg, "ERROR", MB_ICONSTOP);
                }
              else if (debug)
                {
                  wsprintf(cMsg, "RCV-Wait RC = %d, %s.",
                           nReturnCode, (LPSTR)returnstring[nReturnCode]);
                  wsprintf(cBuffer, "\n# bytes received = %d.", receivedLength);
                  lstrcat(cMsg, cBuffer);
                  wsprintf(cBuffer, "\nWhat Received = %d.", what);
                  lstrcat(cMsg, cBuffer);
                  MessageBox(GetFocus(), cMsg, "Host Sent Data", MB_OK);
                }
            } /* end if receive and wait */
 
          if (nReturnCode != EHNAPPC_OK)   /* Break the for loop because */
            break;                    /* the conversation's in a bad way */
 
          /* Verify the data that we received from the host */
          if (receivedLength == CustomRcv.Size)
            {
              for (j=0; j < receivedLength-LL_HDR; j++)
                {
                  if (CustomPtr->data[j] != RcvPtr->data[j])
                    break;
                }
              if (j == receivedLength-LL_HDR)
                {
                  wsprintf(cBuffer, "Custom Data for LL #%d is OK!", i+1);
                  MessageBox(GetFocus(), cBuffer, "OK", MB_ICONINFORMATION);
                }
              else
                {
                  wsprintf(cMsg, "Custom Data error at position %d!", j+1);
                  lstrcat(cMsg, "\nHost did not send the data correctly!");
                  MessageBox(GetFocus(), cMsg, "ERROR", MB_OK);
                }
            }
          else
            {
              wsprintf(cBuffer,"Custom Data error! Only received %d bytes.",
                       receivedLength);
              MessageBox(GetFocus(), cBuffer, "ERROR", MB_OK);
            }
         } /* end for */
 
        /* Must issue another receive to get Send indicator. */
        nReturnCode = EHNAPPC_ReceiveAndWait(hWnd, conv_id,
                                             EHNAPPC_LL,
                                             0,
                                             (LPVOID) &command,
                                             (LPBYTE) &what,
                                             (LPBYTE) &reqsToSend,
                                             (LPWORD) &receivedLength);
 
        MessageBox(GetFocus(), "Custom data option completed.",
                   "Command Result", MB_OK);
 
        GlobalUnlock(hTempRcv);
        GlobalFree(hTempRcv);      /* free up memory */
 
        if (rmTemp.dwBytes > 0)
          //GlobalDosFree(rmTemp.wSelector);
			
        break;
 
 
      case IDM_FLIPSTATE:
        if (! SetCONID(hWnd))
          break;
        nReturnCode = DO_Flip(hWnd);
        wsprintf(cBuffer,"Flip return code = %d, %s",
                 nReturnCode, (LPSTR)returnstring[nReturnCode]);
        MessageBox(GetFocus(), cBuffer, "Command Result", MB_OK);
        break;
 
      case IDM_RESTORESTATE:
        if (! SetCONID(hWnd))
          break;
        nReturnCode = DO_Restore(hWnd);
        wsprintf(cBuffer,"Restore return code = %d, %s",
                 nReturnCode, (LPSTR)returnstring[nReturnCode]);
        MessageBox(GetFocus(), cBuffer, "Command Result", MB_OK);
        break;
 
      case IDM_SETCONVID:
        SetCONID(hWnd);
        break;
 
      case IDM_DEBUG:
        debug = !debug;
        if (debug)
          {
            MessageBox(GetFocus(), "DEBUG SWITCH ON!", "DEBUG INFO!", MB_OK);
            CheckMenuItem(GetMenu(hWnd), wParam, MF_CHECKED);
          }
        else
          {
            MessageBox(GetFocus(), "DEBUG SWITCH OFF!", "DEBUG INFO!", MB_OK);
            CheckMenuItem(GetMenu(hWnd), wParam, MF_UNCHECKED);
          }
        break;
 
      case IDM_ABOUT:
        DialogBox(hInst, "DB_ABOUTBOX", hWnd, lpfnAboutDlg);
        break;
 
      case IDM_EXIT:
        PostMessage(hWnd, WM_CLOSE, NULL, NULL);
        break;
 
      } /* end switch (wParam) */
      break;
 
 
    case WM_CLOSE:
    case WM_DESTROY:
      FreeProcInstance(lpfnSystemList);
      FreeProcInstance(lpfnFullSystemList);
      FreeProcInstance(lpfnAllocateInfoDlg);
      FreeProcInstance(lpfnExtAllocateInfoDlg);
      FreeProcInstance(lpfnDeallocateDlg);
      FreeProcInstance(lpfnGetUserIDDlg);
      FreeProcInstance(lpfnTakeMemDlg);
      FreeProcInstance(lpfnCustomDataDlg);
      FreeProcInstance(lpfnCustomRcvDlg);
      FreeProcInstance(lpfnA2eDlg);
      FreeProcInstance(lpfnLocationDlg);
      FreeProcInstance(lpfnGetClistDlg);
      FreeProcInstance(lpfnAboutDlg);
 
      for (i=0; i < MAX_CONVERSATIONS; ++i)
        {
          if (convlistarray[i].allocated == TRUE)
            EHNAPPC_Deallocate(hWnd, conv_id, EHNAPPC_DEALLOCATEABEND);
        }
 
      //if (rmAllocated.dwBytes > 0L)
        //GlobalDosFree(rmAllocated.wSelector);
		
      GlobalUnlock(hpattern1);
      GlobalFree(hpattern1);
      GlobalUnlock(hDataPtr);
      GlobalFree(hDataPtr);
      PostQuitMessage(0);
      break;
 
    default:
      return DefWindowProc(hWnd, wMsgID, wParam, lParam);
    } /* end switch wMsgID */
  return 0;
}
 
 
//*------------------------------------------------------------------------
//* Dialog box procedures.  Note: GetParent is used to send the window
//* handle from the OverlappedWindowProc1 procedure to the PC Support/400
//* APIs. This is done for the sake of PCSMON (the PC Support/400 message
//* spy tool on QIWSTOOL).  It's not necessary, but it allows you to be
//* more selective when you monitor your Windows applications that use
//* the PC Support/400 APIs.
//*------------------------------------------------------------------------
 
//*------------------------------------------------------------------------
//| AllocateInfoDlgProc
//|     Parameters:
//|         hWnd    - Handle to Window which message is delivered to.
//|         msgID   - ID number of message
//|         wParam  - 16-bit parameter
//|         lParam  - 32-bit parameter
//|
//|     This function displays the Allocate dialog box and issues the
//|     Allocate or Extended Allocate verb.
//*------------------------------------------------------------------------
BOOL FAR PASCAL AllocateInfoDlgProc (HWND     hWnd,
                                     WORD     wMsgID,
                                     WORD     wParam,
                                     LONG     lParam)
{
  char     cBuffer[100];
  int      nReturnCode;
  int      i;
  char     what;                       /* What received from server      */
  char     reqsToSend;                 /* Output Request to send field   */
  int      receivedLength;             /* Length of received data        */
  int      bytecount;
  LPVOID   pipptr;
  REALMEM  rmPipData;
  unsigned int commbuffsize;
  static SyncLevelEnum syncind;
  static MemoryEnum piplocation;
  static char tpname[20] = "QGPL/RTRW400";
  static char hostname[EHNAPPC_SYSNAME_LENGTH] = "";
  static char commbuff[10] = "271";
 
  switch (wMsgID)
  {
  case WM_INITDIALOG:
    CheckRadioButton(hWnd, IDE_SYNCNONE, IDE_SYNCCONFIRM, IDE_SYNCNONE);
    syncind = EHNAPPC_SYNCLEVELNONE;
    CheckRadioButton(hWnd, IDE_EXTENDED, IDE_REAL, IDE_EXTENDED);
    piplocation = EXTENDED;
    SetDlgItemText(hWnd,IDE_COMMBUFF, commbuff);
    SetDlgItemText(hWnd,IDE_HOSTNAME, hostname);
    SetDlgItemText(hWnd,IDE_TPNAME, tpname);
    SetDlgItemText(hWnd,IDE_PIPDATA,"");
    SetFocus (GetDlgItem (hWnd, IDE_COMMBUFF));
    break;
 
  case WM_COMMAND:
    switch (wParam)
    {
    case IDE_SYNCNONE:
    case IDE_SYNCCONFIRM:
      CheckRadioButton(hWnd, IDE_SYNCNONE, IDE_SYNCCONFIRM, wParam);
      if (wParam == IDE_SYNCNONE)
        syncind = EHNAPPC_SYNCLEVELNONE;
      else
        syncind = EHNAPPC_SYNCLEVELCONFIRM;
      return TRUE;
 
    case IDE_EXTENDED:
    case IDE_REAL:
      CheckRadioButton(hWnd, IDE_EXTENDED, IDE_REAL, wParam);
      piplocation = (wParam == IDE_EXTENDED) ? EXTENDED : REAL;
	  piplocation = EXTENDED;  //dxd
      return TRUE;
 
    case IDOK:
      GetDlgItemText(hWnd,IDE_HOSTNAME,hostname, sizeof(hostname)+1);
      GetDlgItemText(hWnd, IDE_TPNAME, tpname, sizeof(tpname)+1);
      GetDlgItemText(hWnd,IDE_COMMBUFF, commbuff, sizeof(commbuff)+1);
      commbuffsize = atoi(commbuff);
 
      pipout[0] = 0;
      pipout[1] = 0;
      GetDlgItemText(hWnd, IDE_PIPDATA, pipdata, sizeof(pipdata)+1);
      if (debug)
        {
          wsprintf (cBuffer, "PIP DATA LENGTH = %d", lstrlen(pipdata));
          MessageBox(GetFocus(), cBuffer, "PIP DATA", MB_OK);
        }
 
      if (lstrlen(pipdata) != 0) /* Got some pip data?        */
        {                        /* Fill in the pip structure */
          pipout[0] = 00;
          pipout[1] = lstrlen(pipdata) + 8; /* str size + overhead */
          pipout[2] = 0x12;
          pipout[3] = 0xF5;
          pipout[4] = 00;
          pipout[5] = pipout[1] - 4;
          pipout[6] = 0x12;
          pipout[7] = 0xE2;
          memcpy(pipout+8, pipdata, lstrlen(pipdata));
          bytecount = sizeof(pipout) - 8;
          EHNDT_ASCIIToEBCDIC(GetParent(hWnd),
                              pipout+8,
                              pipout+8,
                              lstrlen(pipdata),
                              &bytecount);
        }
 
      /* Determine whether the pip data is supposed to be in */
      /* extended or conventional memory.                    */
      if (piplocation == EXTENDED)   /* Use extended memory  */
        {
          pipptr = (LPVOID) pipout;
          rmPipData.dwBytes = 0L;
        }
      else                       /* Move pip data to real memory */
        {
          if (! GlobalDosLargest(sizeof(pipout), sizeof(pipout), &rmPipData))
            {
              MessageBox(GetFocus(),
                   "Failed to allocate memory for pip data.",
                   "TEST DRIVER MEMORY ALLOCATION ERROR", MB_ICONSTOP);
              EndDialog(hWnd, FALSE);
              return FALSE;
            }
          pipptr = (LPVOID) MAKELONG(0, rmPipData.wSelector);
          memcpy(pipptr, pipout, sizeof(pipout));
        }
 
      EndDialog(hWnd,TRUE);
 
      if (AllocateVerb)
        {
          globalret = EHNAPPC_Allocate(GetParent(hWnd),
                                       commbuffsize,
                                       EHNAPPC_BASIC,
                                       syncind,
                                       (LPSTR) hostname,
                                       (LPSTR) tpname,
                                       (int) pipout[1],  /* PIP data */
                                       (LPVOID) pipptr,
                                       (LPDWORD) &conv_id);
        }
      else /* The Extended Allocate option was selected */
        {
          globalret = EHNAPPC_ExtendedAllocate(GetParent(hWnd),
                                       commbuffsize,
                                       EHNAPPC_BASIC,
                                       syncind,
                                       (LPSTR) hostname,
                                       (LPSTR) tpname,
                                       (LPSTR) modename,
                                       sectype,
                                       (LPSTR) userid,
                                       (LPSTR) password,
                                       (int) pipout[1],  /* PIP data */
                                       (LPVOID) pipptr,
                                       (LPDWORD) &conv_id);
        }
 
      /* Free the pip data memory if any */
      //if (rmPipData.dwBytes > 0L)
        //GlobalDosFree(rmPipData.wSelector);
	    
 
      if (globalret != EHNAPPC_OK)     /* Bad return code?  */
        return(TRUE);                  /* Yes, return       */
 
      for (i=0; i < MAX_CONVERSATIONS; i++)  /* add conversation to table */
        {
          if (convlistarray[i].allocated == FALSE)
            { /* add to table -- found a slot ! */
              convlistarray[i].allocated = TRUE;
              wsprintf((LPSTR) &convlistarray[i].convhandle,
                       "%lx", conv_id);
              convlistarray[i].convhex = conv_id;
              break;
            }  /* end if block */
        } /* end for */
 
      if (debug)
        {
          wsprintf(cBuffer,"Allocate ret code = %d, %s",
                   globalret, (LPSTR)returnstring[globalret]);
          MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
        }
 
      wsprintf (cBuffer,"CONV ID = %10lx", conv_id);
      MessageBox(GetFocus(), cBuffer, "CONV ID INFO", MB_OK);
 
      globalret = EHNAPPC_PrepareToReceive(GetParent(hWnd), conv_id);
      if (debug)
        {
          wsprintf(cBuffer, "Prepare to receive RC = %d, %s",
                   globalret, (LPSTR)returnstring[globalret]);
          MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
        }
      if (globalret != EHNAPPC_OK)     /* Bad return code?  */
        {
          return(TRUE);                /* Yes, return       */
        }
 
      do   /* Receive the pip data back */
        {
          globalret = EHNAPPC_ReceiveImmediate(GetParent(hWnd),
                                               conv_id,
                                               EHNAPPC_LL,
                                               168,
                                               (LPVOID) &command,
                                               (LPBYTE) &what,
                                               (LPBYTE) &reqsToSend,
                                               (LPWORD) &receivedLength);
        } while (globalret == EHNAPPC_UNSUCCESSFUL);
 
      if (debug)
        {
          wsprintf(cBuffer,"Rcv Immediate RC = %d, %s",
                   globalret, (LPSTR)returnstring[globalret]);
          MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
        }
      if (globalret != EHNAPPC_OK)     /* Bad return code?  */
        {
          return(TRUE);
        }
 
      /* Must issue second receive to get Send indicator.  */
      globalret = EHNAPPC_ReceiveAndWait(GetParent(hWnd),
                                         conv_id,
                                         EHNAPPC_LL,
                                         0,
                                         (LPVOID) &command,
                                         (LPBYTE) &what,
                                         (LPBYTE) &reqsToSend,
                                         (LPWORD) &receivedLength);
      if (debug)
        {
          wsprintf(cBuffer, "Rcv and wait RC = %d, %s",
                   globalret, (LPSTR)returnstring[globalret]);
          MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
        }
 
      return(TRUE);         /* return back to main */
 
    case IDCANCEL:
      EndDialog(hWnd,FALSE);
      return FALSE;
    } /* end switch (wParam) */
  } /* end switch (wMsgID) */
  return FALSE;
}
 
 
//*------------------------------------------------------------------------
//| ExtAllocateInfoDlgProc
//|     Parameters:
//|         hWnd    - Handle to Window which message is delivered to.
//|         msgID   - ID number of message
//|         wParam  - 16-bit parameter
//|         lParam  - 32-bit parameter
//|
//|     This dialog box prompts for the Extended Allocate parameters.
//*------------------------------------------------------------------------
BOOL FAR PASCAL ExtAllocateInfoDlgProc (HWND     hWnd,
                                        WORD     wMsgID,
                                        WORD     wParam,
                                        LONG     lParam)
{
  switch (wMsgID)
  {
  case WM_INITDIALOG:
    SetFocus(GetDlgItem (hWnd, IDM_EXT_USERID));  /* Set the defaults */
    SetDlgItemText(hWnd, IDM_EXT_MODE, "QPCSUPP");
    CheckRadioButton(hWnd, IDM_EXT_NONE, IDM_EXT_PGM, IDM_EXT_PGM);
    sectype = EHNAPPC_SECURITY_PGM;
    return TRUE;
 
  case WM_COMMAND:
    switch (wParam)
    {
    case IDOK:
      GetDlgItemText(hWnd,IDM_EXT_USERID, userid,   (sizeof(userid)+1));
      GetDlgItemText(hWnd,IDM_EXT_PASS,   password, (sizeof(password)+1));
      GetDlgItemText(hWnd,IDM_EXT_MODE,   modename, (sizeof(modename)+1));
      EndDialog(hWnd,TRUE);
      return TRUE;
 
    case IDCANCEL:
      EndDialog(hWnd,FALSE);
      return FALSE;
 
    case IDM_EXT_NONE:
      sectype = EHNAPPC_SECURITY_NONE;
      return TRUE;
 
    case IDM_EXT_SAME:
      sectype = EHNAPPC_SECURITY_SAME;
      return TRUE;
 
    case IDM_EXT_PGM:
      sectype = EHNAPPC_SECURITY_PGM;
      return TRUE;
    }
  }
  return FALSE;
}
 
//*------------------------------------------------------------------------
//| SystemListProc
//|     Parameters:
//|         hWnd    - Handle to Window which message is delivered to.
//|         msgID   - ID number of message
//|         wParam  - 16-bit parameter
//|         lParam  - 32-bit parameter
//|
//|     This box displays the list of active systems.
//*------------------------------------------------------------------------
BOOL FAR PASCAL SystemListProc (HWND     hWnd,
                                WORD     wMsgID,
                                WORD     wParam,
                                LONG     lParam)
{
  char            cBuffer[100];
  HANDLE          hMem;
  HWND            hCtl;
  LPSTR           lpSystemName;
  int             nReturnCode;
  static int      isyscount, systemcount;     /* These vars must be static */
  static struct   AS400_Sys systems;          /*   or else...              */
  LPSYSSTRUC      systemlist;
  int             syslen;
 
  switch (wMsgID)
  {
  case WM_INITDIALOG:
    hCtl = GetDlgItem(hWnd, ID_SYSLISTBOX); /* Get handle */
    hMem = GlobalAlloc(GMEM_LOWER | GMEM_MOVEABLE, 16);
    lpSystemName = GlobalLock(hMem);
 
    systemlist = &systems;
    nReturnCode = EHNAPPC_QuerySystems(GetParent(hWnd),
                                       &systemcount,
                                       systemlist);
    if (nReturnCode == EHNAPPC_OK)
      {
        if (systemcount > 0)         /* Mark the default system */
          {
          syslen = lstrlen(systems.EHNAPPC_SysName[0]);
          systems.EHNAPPC_SysName[0][syslen] = '*';
          systems.EHNAPPC_SysName[0][syslen + 1] = '\0';
          }
        for (isyscount=0; isyscount <= (systemcount-1); isyscount++)
         {
           memcpy(lpSystemName,
                    systemlist->EHNAPPC_SysName[isyscount],
                    EHNAPPC_SYSNAME_LENGTH); /* sys name to window */
           SendMessage(hCtl, LB_ADDSTRING, NULL, (DWORD) lpSystemName);
         }
      }
    else
      {
        SetFocus(hCtl);
        wsprintf(cBuffer, "QuerySystems return code = %d, %s",
                 nReturnCode, (LPSTR)returnstring[nReturnCode]);
        MessageBox(GetFocus(), cBuffer, "Command Result", MB_OK);
      }
 
    GlobalFree(hMem);
    SetFocus(hCtl);
    return TRUE;
 
  case WM_COMMAND:
    switch (wParam)
    {
    case IDOK:
      EndDialog(hWnd, TRUE);
      break;
 
    default:
      break;
    }
  }
  return FALSE;
}
 
//*------------------------------------------------------------------------
//| FullSystemListProc
//|     Parameters:
//|         hWnd    - Handle to Window which message is delivered to.
//|         msgID   - ID number of message
//|         wParam  - 16-bit parameter
//|         lParam  - 32-bit parameter
//|
//|     This box displays the list of active systems.
//*------------------------------------------------------------------------
BOOL FAR PASCAL FullSystemListProc (HWND     hWnd,
                                    WORD     wMsgID,
                                    WORD     wParam,
                                    LONG     lParam)
{
  char            cBuffer[100];
  HANDLE          hMem;
  HWND            hCtl;
  LPSTR           lpSystemName;
  int             nReturnCode;
  static int      isyscount, systemcount;     /* These vars must be static */
  static struct   AS400_FullSys systems;      /*   or else...              */
  LPFULLSYSSTRUC  systemlist;
  int             syslen, aliaslen, netlen;
  int             tabs[2];
 
  switch (wMsgID)
  {
  case WM_INITDIALOG:
    hCtl = GetDlgItem(hWnd, BOX_FULLSYSPRES); /* Get handle */
    hMem = GlobalAlloc(GMEM_LOWER | GMEM_MOVEABLE, 32); /* 32 Bytes for FullName */
    lpSystemName = GlobalLock(hMem);
 
    systemlist = &systems;
    nReturnCode = EHNAPPC_QueryFullSystems(GetParent(hWnd),
                                       &systemcount,
                                       systemlist);
    if (nReturnCode == EHNAPPC_OK)
      {
        tabs[0] = 55;
        tabs[1] = 110;
        SendMessage(hCtl, LB_SETTABSTOPS, 2,
                    (DWORD) (LPINT) tabs); /* Tab stops */
 
        if (systemcount > 0)         /* Mark the default system */
          {
          syslen = lstrlen(systems.FullName[0].EHNAPPC_SysName);
          systems.FullName[0].EHNAPPC_SysName[syslen] = '*';
          systems.FullName[0].EHNAPPC_SysName[syslen + 1] = '\0';
          }
        for (isyscount=0; isyscount <= (systemcount-1); isyscount++)
         {
           aliaslen = lstrlen(systemlist->
                              FullName[isyscount].EHNAPPC_AliasName);
           memcpy(lpSystemName,
                    systemlist->FullName[isyscount].EHNAPPC_AliasName,
                    aliaslen);                        /* Alias name to window */
           *(lpSystemName + aliaslen) = TAB;          /* put the tab */
 
           netlen = lstrlen(systemlist->
                            FullName[isyscount].EHNAPPC_NetworkName);
           memcpy(lpSystemName + aliaslen + 1,
                    systemlist->FullName[isyscount].EHNAPPC_NetworkName,
                    netlen);                        /* Network name to window */
           *(lpSystemName + aliaslen + 1 + netlen) = TAB;
 
           memcpy(lpSystemName + aliaslen + netlen + 2,
                    systemlist->FullName[isyscount].EHNAPPC_SysName,
                    EHNAPPC_SYSNAME_LENGTH); /* System name to window*/
           SendMessage(hCtl, LB_ADDSTRING, NULL, (DWORD) lpSystemName);
         }
      }
    else
      {
        SetFocus(hCtl);
        wsprintf(cBuffer, "QueryFullSystems return code = %d, %s",
                 nReturnCode, (LPSTR)returnstring[nReturnCode]);
        MessageBox(GetFocus(), cBuffer, "Command Result", MB_OK);
      }
 
    GlobalFree(hMem);
    SetFocus(hCtl);
    return TRUE;
 
  case WM_COMMAND:
    switch (wParam)
    {
    case IDOK:
      EndDialog(hWnd, TRUE);
      break;
 
    default:
      break;
    }
  }
  return FALSE;
}
 
//*------------------------------------------------------------------------
//| DeaAllocateDlgProc
//|     Parameters:
//|         hWnd    - Handle to Window which message is delivered to.
//|         msgID   - ID number of message
//|         wParam  - 16-bit parameter
//|         lParam  - 32-bit parameter
//|
//|     This dialog box prompts for the Deallocate parameters.
//*------------------------------------------------------------------------
BOOL FAR PASCAL DeallocateDlgProc (HWND     hWnd,
                                   WORD     wMsgID,
                                   WORD     wParam,
                                   LONG     lParam)
{
  switch (wMsgID)
  {
  case WM_INITDIALOG:
    CheckRadioButton(hWnd, IDC_SYNCHLEVEL, IDC_ABEND, IDC_SYNCHLEVEL);
    SetFocus(GetDlgItem(hWnd, IDC_SYNCHLEVEL));
    break;
 
  case WM_COMMAND:
    switch (wParam)
    {
    case IDOK:
      EndDialog(hWnd,TRUE);
      return TRUE;
 
    case IDCANCEL:
      EndDialog(hWnd,FALSE);
      return FALSE;
 
    case IDC_ABEND:
      dealloctype = EHNAPPC_DEALLOCATEABEND;
      return TRUE;
 
    case IDC_FLUSH:
      dealloctype = EHNAPPC_DEALLOCATEFLUSH;
      return TRUE;
 
    case IDC_SYNCHLEVEL:
      dealloctype = EHNAPPC_DEALLOCATESYNCLEVEL;
      return TRUE;
    }
  }
  return FALSE;
}
 
//*------------------------------------------------------------------------
//| GetUserIDDlgProc
//|     Parameters:
//|         hWnd    - Handle to Window which message is delivered to.
//|         msgID   - ID number of message
//|         wParam  - 16-bit parameter
//|         lParam  - 32-bit parameter
//|
//|     This dialog box prompts for a system to see which userid was
//|     used to connect to that system.  If the user doesn't specify a
//|     system, the default system is queried.
//*------------------------------------------------------------------------
BOOL FAR PASCAL GetUserIDDlgProc(HWND     hWnd,
                                 WORD     wMsgID,
                                 WORD     wParam,
                                 LONG     lParam)
{
  char  cBuffer[100];
  char  cUserId[10];
  char  cSystem[20];
  int   nReturnCode;
 
  switch (wMsgID)
  {
  case WM_INITDIALOG:
    SetDlgItemText(hWnd, IDC_SNAME, "");
    SetFocus(GetDlgItem(hWnd, IDC_SNAME));
    break;
 
  case WM_COMMAND:
    switch (wParam)
    {
    case IDOK:
      GetDlgItemText(hWnd, IDC_SNAME, cSystem, (sizeof(cSystem)+1));
      if (debug)
        {
          wsprintf(cBuffer, "System name = '%s'", (LPSTR)cSystem);
          MessageBox(GetFocus(), cBuffer, "System Name", MB_OK);
        }
 
      nReturnCode = EHNAPPC_QueryUserId(GetParent(hWnd),
                                        (LPSTR) cSystem,
                                        (LPSTR) &cUserId);
      if (nReturnCode == EHNAPPC_OK)
        {
          wsprintf(cBuffer,"USER ID = %s", (LPSTR)cUserId);
          MessageBox(GetFocus(), cBuffer, "INFO", MB_OK);
        }
      else
        {
          wsprintf(cBuffer,"Query User ID RC = %d, %s",
                   nReturnCode, (LPSTR)returnstring[nReturnCode]);
          MessageBox(GetFocus(), cBuffer, "ERROR", MB_OK);
        }
      EndDialog(hWnd,TRUE);
      return TRUE;
    }
  }
  return FALSE;
}
 
//*------------------------------------------------------------------------
//| TakeMemDlgProc
//|     Parameters:
//|         hWnd    - Handle to Window which message is delivered to.
//|         msgID   - ID number of message
//|         wParam  - 16-bit parameter
//|         lParam  - 32-bit parameter
//|
//|     Prompts for the amount of conventional memory to take away.
//|     The option is used to reduce conventional memory to make sure
//|     low memory situations are handled okay.
//*------------------------------------------------------------------------
BOOL FAR PASCAL TakeMemDlgProc (HWND     hWnd,
                                WORD     wMsgID,
                                WORD     wParam,
                                LONG     lParam)
{
  char   cBuffer[50];
  DWORD  dwMin, dwMax;
 
  switch (wMsgID)
  {
  case WM_INITDIALOG:
    return TRUE;
 
  case WM_COMMAND:
    switch (wParam)
    {
    case IDOK:
      GetDlgItemText(hWnd, IDM_AMOUNT_TO_TAKE, cBuffer, (sizeof(cBuffer)));
      EndDialog(hWnd,TRUE);
      if (cBuffer[0] == '*')    /* Go for it all */
        {
          dwMax = MAX_MEM;
          dwMin = MIN_MEM;
        }
      else
        {
          dwMax = atol(cBuffer);
          dwMin = dwMax;
        }
 
      if (GlobalDosLargest(dwMax, dwMin, &rmAllocated))
        {
          wsprintf(cBuffer, "%lu bytes allocated successfully.",
                   rmAllocated.dwBytes);
          MessageBox(GetFocus(), cBuffer, "Take memory", MB_OK);
        }
      else
        {
          MessageBox(GetFocus(),
                     "Failed to allocate the memory. You may have asked for too much.",
                     "Take memory", MB_OK);
          rmAllocated.dwBytes = 0L;
        }
      return TRUE;
 
    case IDCANCEL:
      EndDialog(hWnd,FALSE);
      return FALSE;
    }
  }
  return FALSE;
}
 
//*------------------------------------------------------------------------
//|     CustomDataDlgProc
//|     Parameters:
//|         hWnd    - Handle to Window which message is delivered to.
//|         msgID   - ID number of message
//|         wParam  - 16-bit parameter
//|         lParam  - 32-bit parameter
//|
//*------------------------------------------------------------------------
BOOL FAR PASCAL CustomDataDlgProc(HWND     hWnd,
                                  WORD     wMsgID,
                                  WORD     wParam,
                                  LONG     lParam)
{
  char  cBuffer[200];
  int   i, total;
 
  switch (wMsgID)
  {
  case WM_INITDIALOG:
    CustomLLs.NumOfLLs = 1;  /* Default */
    CheckRadioButton(hWnd, ID_LL1NUM, ID_LL4NUM, ID_LL1NUM);
    CheckDlgButton(hWnd, ID_MEMLOCEXT, 1);
    SetFocus (GetDlgItem (hWnd, ID_LL1NUM));
    break;
 
  case WM_COMMAND:
    switch (wParam)
    {
    case ID_LL1NUM:
      CustomLLs.NumOfLLs = 1;
      return TRUE;
    case ID_LL2NUM:
      CustomLLs.NumOfLLs = 2;
      return TRUE;
    case ID_LL3NUM:
      CustomLLs.NumOfLLs = 3;
      return TRUE;
    case ID_LL4NUM:
     CustomLLs.NumOfLLs = 4;
     return TRUE;
 
    case IDOK:
      GetDlgItemText(hWnd, ID_BYTETOTAL, cBuffer, (sizeof(cBuffer)));
      CustomLLs.TotalBytes = atol(cBuffer);
      if (CustomLLs.TotalBytes > 32765)
        {
          MessageBox(GetFocus(), "Max allowed = 32765. Try again.",
                     "Bad arithmetic", MB_ICONINFORMATION);
          return TRUE;
        }
 
      GetDlgItemText(hWnd, ID_LL1SIZE, cBuffer, (sizeof(cBuffer)));
      CustomLLs.LLinfo[0].Size = atol(cBuffer);
      GetDlgItemText(hWnd, ID_LL2SIZE, cBuffer, (sizeof(cBuffer)));
      CustomLLs.LLinfo[1].Size = atol(cBuffer);
      GetDlgItemText(hWnd, ID_LL3SIZE, cBuffer, (sizeof(cBuffer)));
      CustomLLs.LLinfo[2].Size = atol(cBuffer);
      GetDlgItemText(hWnd, ID_LL4SIZE, cBuffer, (sizeof(cBuffer)));
      CustomLLs.LLinfo[3].Size = atol(cBuffer);
 
      for (i=0, total=0; i < CustomLLs.NumOfLLs; ++i)
        {
          if (CustomLLs.LLinfo[i].Size < LL_HDR) /* Must be big enough for */
            {                                    /*   a sub-LL field.      */
              wsprintf(cBuffer, "Each LL Size must be >= %d.", LL_HDR);
              MessageBox(GetFocus(), cBuffer, "ERROR", MB_ICONEXCLAMATION);
              return TRUE;
            }
          total += CustomLLs.LLinfo[i].Size;
        }
      if (total != CustomLLs.TotalBytes)
        {
          lstrcpy(cBuffer, "Individual amounts do not equal overall");
          lstrcat(cBuffer, " total. Try again.\n(Make sure you");
          lstrcat(cBuffer, " selected the correct number of LLs.)");
          MessageBox(GetFocus(),cBuffer,"Bad arithmetic",MB_ICONEXCLAMATION);
          return TRUE;
        }
 
      GetDlgItemText(hWnd, ID_LL1START, cBuffer, (sizeof(cBuffer)));
      CustomLLs.LLinfo[0].Start = LOBYTE(atoi(cBuffer));
      GetDlgItemText(hWnd, ID_LL2START, cBuffer, (sizeof(cBuffer)));
      CustomLLs.LLinfo[1].Start = LOBYTE(atoi(cBuffer));
      GetDlgItemText(hWnd, ID_LL3START, cBuffer, (sizeof(cBuffer)));
      CustomLLs.LLinfo[2].Start = LOBYTE(atoi(cBuffer));
      GetDlgItemText(hWnd, ID_LL4START, cBuffer, (sizeof(cBuffer)));
      CustomLLs.LLinfo[3].Start = LOBYTE(atoi(cBuffer));
 
      if (IsDlgButtonChecked(hWnd, ID_MEMLOCEXT))
        CustomLLs.MemLocation = EXTENDED;
      else
        CustomLLs.MemLocation = REAL;
 
      EndDialog(hWnd,TRUE);
      return TRUE;
 
    case IDCANCEL:
      EndDialog(hWnd,FALSE);
      return FALSE;
    }
  }
  return FALSE;
}
 
//*------------------------------------------------------------------------
//| CustomRcvDlgProc
//|     Parameters:
//|         hWnd    - Handle to Window which message is delivered to.
//|         msgID   - ID number of message
//|         wParam  - 16-bit parameter
//|         lParam  - 32-bit parameter
//|
//|     Prompts the user for parameters used to do custom-built receive
//|     requests.
//*------------------------------------------------------------------------
BOOL FAR PASCAL CustomRcvDlgProc(HWND     hWnd,
                                 WORD     wMsgID,
                                 WORD     wParam,
                                 LONG     lParam)
{
  char  cBuffer[51];
 
  switch (wMsgID)
  {
  case WM_INITDIALOG:
    CheckDlgButton(hWnd, ID_MEMLOCEXT, BST_CHECKED);
    SetFocus (GetDlgItem (hWnd, ID_LLNUMBER));
    break;
 
  case WM_COMMAND:
    switch (wParam)
    {
    case IDOK:
      GetDlgItemText(hWnd, ID_LLNUMBER, cBuffer, (sizeof(cBuffer)));
      CustomRcv.NumOfLLs = atol(cBuffer);
      if ((CustomRcv.NumOfLLs > 256) || (CustomRcv.NumOfLLs <= 0))
        {
          MessageBox(GetFocus(), "Number of LLs must be > 0 and <= 256.",
                     "Bad value", MB_ICONINFORMATION);
          SetFocus (GetDlgItem (hWnd, ID_LLNUMBER));
          return TRUE;
        }
 
      /* Get the size of each LL. This number includes the LL header. */
      GetDlgItemText(hWnd, ID_LLSIZE, cBuffer, (sizeof(cBuffer)));
      CustomRcv.Size = atol(cBuffer);
      if ((CustomRcv.Size > 32767) || (CustomRcv.Size < LL_HDR))
        {
          wsprintf(cBuffer, "LL length must be >= %d and <= 32767.", LL_HDR);
          MessageBox(GetFocus(), cBuffer, "Bad value", MB_ICONINFORMATION);
          SetFocus(GetDlgItem(hWnd, ID_LLSIZE));
          return TRUE;
        }
 
      if (IsDlgButtonChecked(hWnd, ID_MEMLOCEXT))
        CustomRcv.MemLocation = EXTENDED;
      else
        CustomRcv.MemLocation = EXTENDED; //dxd just in case REAL;
 
      EndDialog(hWnd,TRUE);
      return TRUE;
 
    case IDCANCEL:
      EndDialog(hWnd,FALSE);
      return FALSE;
    }
  }
  return FALSE;
}
 
/****************************************************************************
 
    FUNCTION: GetCListDlgProc(HWND, unsigned, WORD, LONG)
 
    PURPOSE:  Prompts the user to select a conversation ID.
 
****************************************************************************/
BOOL FAR PASCAL GetCListDlgProc (HWND     hWnd,
                                 unsigned wMsgID,
                                 WORD     wParam,
                                 LONG     lParam)
{
  HWND  hCtl;
  HANDLE hMem;
  LPSTR lpConvId;
  static BOOL found;
  int i, j;
 
  switch (wMsgID)
  {
  case WM_INITDIALOG:                /* message: initialize dialog box */
    /* Remove all duplicate or deallocated conversation IDs */
    for (i=0; i < MAX_CONVERSATIONS; i++)
     {
       if (convlistarray[i].allocated == TRUE)
         {                          /* See if it's a duplicate */
           for (j=0; j < i; j++)
            {
              if (convlistarray[i].convhex == convlistarray[j].convhex)
                {
                  convlistarray[i].allocated = FALSE;
                  convlistarray[i].convhex   = 0L;
                }
            }
         }
       if (convlistarray[i].allocated == TRUE)
         {                          /* See if it's still valid */
           if (EHNAPPC_QueryConvState(GetParent(hWnd),
                       convlistarray[i].convhex) == EHNAPPC_RESET_STATE)
             {
               convlistarray[i].allocated = FALSE;
               convlistarray[i].convhex   = 0L;
             }
         }
      } /* end for */
 
    hCtl = GetDlgItem(hWnd, ID_NEWCLISTBOX); /* Get handle */
    hMem = GlobalAlloc(GMEM_LOWER | GMEM_MOVEABLE, 16);
    lpConvId = GlobalLock(hMem);
 
    for (i=0; i < MAX_CONVERSATIONS; i++)
     {
       if (convlistarray[i].allocated == TRUE)
         {
           memcpy(lpConvId,
                    convlistarray[i].convhandle,
                    9);  /* convid in ascii */
           SendMessage(hCtl, LB_ADDSTRING, NULL, (DWORD)lpConvId);
         } /* end if */
      } /* end for */
    GlobalFree(hMem);
    SetFocus(hCtl);
    return TRUE;
 
  case WM_COMMAND:                      /* message: received a command */
    switch (wParam)
      {
      case IDOK:                        /*  "OK" selected              */
        hCtl = GetDlgItem(hWnd,ID_NEWCLISTBOX );       /* Get handle   */
        convindex  = SendMessage(hCtl, LB_GETCURSEL,
                                 0, 0L); /*  Index of selected conv id */
 
        if (convindex >= MAX_CONVERSATIONS)
          {
            if (debug)
              MessageBox(GetFocus(),
                     "No session chosen, will use previously selected ID.",
                     "Session Info", MB_OK);
            EndDialog(hWnd, TRUE);          /* Exits the dialog box */
            break; /* out of case */
          }
 
        /* Find the selected entry in the array. */
        i = 0; /* to traverse thru the array */
        j = 0; /* to keep track of # of allocated sessions */
        found = FALSE;
        do {
          if (convlistarray[i].allocated == TRUE)
            {
              j++ ;  /* found an allocated conversation -- inc counter */
              if (j > convindex)
                found = TRUE;  /* found what we need */
            }
           i++ ;
         } while (! found);
 
        conv_id = convlistarray[i-1].convhex;
        convindex = i-1;             /* Set up index for rest of world */
        EndDialog(hWnd, TRUE);       /* Exits the dialog box           */
        break;
 
      case IDCANCEL:                 /* "Cancel" box selected          */
        EndDialog(hWnd, FALSE);      /* Exits the dialog box           */
        return (FALSE);
      }
 
      default:
        break;
  }
 
  return FALSE;                         /* Didn't process a message    */
}
 
//*------------------------------------------------------------------------
//| A2eDlgProc
//|     Parameters:
//|         hWnd    - Handle to Window which message is delivered to.
//|         msgID   - ID number of message
//|         wParam  - 16-bit parameter
//|         lParam  - 32-bit parameter
//|
//|     This dialog box prompts the user for data which will be translated
//|     to EBCDIC.
//*------------------------------------------------------------------------
BOOL FAR PASCAL A2eDlgProc(HWND     hWnd,
                           WORD     wMsgID,
                           WORD     wParam,
                           LONG     lParam)
{
  switch (wMsgID)
  {
  case WM_INITDIALOG:
    SetFocus (GetDlgItem (hWnd, ID_A2ETEXT));
    break;
 
  case WM_COMMAND:
    switch (wParam)
    {
    case IDOK:
      GetDlgItemText(hWnd, ID_A2ETEXT, AsciiBuffer, (sizeof(AsciiBuffer)));
      EndDialog(hWnd,TRUE);
      return TRUE;
 
    case IDCANCEL:
      EndDialog(hWnd,FALSE);
      return FALSE;
    }
  }
  return FALSE;
}
 
//*------------------------------------------------------------------------
//| LocationDlgProc
//|     Parameters:
//|         hWnd    - Handle to Window which message is delivered to.
//|         msgID   - ID number of message
//|         wParam  - 16-bit parameter
//|         lParam  - 32-bit parameter
//|
//|     This dialog box prompts the user for the location where a buffer
//|     should be allocated (conventional or extended memory).  This
//|     option is needed for testing.
//*------------------------------------------------------------------------
BOOL FAR PASCAL LocationDlgProc(HWND     hWnd,
                                WORD     wMsgID,
                                WORD     wParam,
                                LONG     lParam)
{
  switch (wMsgID)
  {
  case WM_INITDIALOG:
    CheckDlgButton(hWnd, ID_EXTENDED, BST_CHECKED);  //1
    SetFocus(GetDlgItem(hWnd, IDOK));
    break;
 
  case WM_COMMAND:
    switch (wParam)
    {
    case IDOK:
      if (IsDlgButtonChecked(hWnd, ID_EXTENDED))
        Location = EXTENDED;
      else
        Location = REAL;
 
      EndDialog(hWnd,TRUE);
      return TRUE;
 
    case IDCANCEL:
      EndDialog(hWnd,FALSE);
      return FALSE;
    }
  }
  return FALSE;
}
 
//*------------------------------------------------------------------------
//| AboutDlgProc
//|     Parameters:
//|         hWnd    - Handle to Window which message is delivered to.
//|         msgID   - ID number of message
//|         wParam  - 16-bit parameter
//|         lParam  - 32-bit parameter
//|
//|     Display the About box.
//*------------------------------------------------------------------------
BOOL FAR PASCAL AboutDlgProc(HWND       hWnd,
                             unsigned   message,
                             WORD       wParam,
                             LONG       lParam)
{
  switch (message)
  {
  case WM_INITDIALOG:
    return (TRUE);
 
  case WM_COMMAND:
    if (wParam == IDOK || wParam == IDCANCEL)
      {
        EndDialog(hWnd, TRUE);
        return (TRUE);
      }
    break;
  }
  return (FALSE);
}
